# Restart with tcl: -*- mode: tcl; tab-width: 8; -*- 
#************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: audioPlay.tcl,v 1.13 2005/10/28 08:40:22 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# eallaert  2005-02-15  first version
#
# audioPlay.tcl, audioServer.tcl and audioCtrl.tcl were "inspired" by some
# of the snack demos, in particular rplay.tcl and aserver.tcl.

#************************************************************************
#   NAME
#   audioPlay - play a soundfile
#   
#   SYNOPSIS
#   \fBaudioPlay\fR \fI?-volume <volume>? <soundFile>\fR
#   
#   DESCRIPTION
#   audioPlay will play a <soundFile>. If the \fI-volume\fR option is specified,
#   the gain will first be adjusted to <volume> (on a scale from 0 to 100); in
#   the latter case (option \fI-volume\fR specified) the audio-server will also
#   be un-muted if it was muted; after playing the <soundFile>, the volume will
#   be set back to its original setting.
#
#   The supported formats of <soundFile> are of course determined by 
#   the sound server. For the Snack-based server this is WAV, AU, AIFF, MP3,
#   CSL, SND, SD, SMP, RAW and NIST/Sphere. For the HP audio-server, this is
#   AU, SND, WAV and RAW. 
#
#   The list of supported servers is given by the macro \fISUPPORT\fR, which is
#   set in the Makefile. It can be any list of the words \fISnack\fR, \fIHP\fR and
#   \fINCD\fR. Apart from specifying which audio-servers are supported, it also
#   sets the order in which they will be tested in case a sound is sent to
#   a not yet known audio-server.
#   
#   The following strategy is applied trying to determine the nature of the 
#   various types of sound servers:
#   1. First, the host of the server is given by the AUDIO/DISPLAY 
#      environment variable (see below).
#   2. For Snack, try to connect to an audioServer. These servers are
#      listening on a particular TCP/IP port, as defined by the macro \fIPORT\fR.
#      If this fails, the audioServer is not (anymore) active.
#   3. For NCD, and if the client process is running on HP-UX, try the 
#      \fIlocal\fR client application for NCD X-terminals. The path to this client 
#      program is given in the Makefile by the variable \fINCDPATH\fR.
#   4. For HP, and if the client process is running on HP-UX, try directly 
#      the \fIlocal\fR snack Tcl-extension, which uses the HP audio library.
#   
#   This determination of the server type takes place the first time a sound
#   file is sent to a particular server. If none of the types fit, subsequent
#   calls to this procedure will only look for an audioServer. 
#   
#   Remark that the check for an HP server will take ~30 seconds if 
#   \fIAUDIO/DISPLAY\fR is pointing to an HP box without audio hardware (or with
#   the Aserver not active). However, if audioPlay is used as a procedure,
#   this will only be for the first attempt to play a sound on that server.
#   Subsequent attempts will fail immediately, as only the existence of an
#   active audioServer will be verified - see above.
#   
#   FILES
#   
#   ENVIRONMENT
#   The environment variable \fIAUDIO\fR is used to determine the hostname of the
#   audio-server. It can be set to a hostname or IP address. If \fIAUDIO\fR is not
#   set, the environment variable \fIDISPLAY\fR will be used (i.e. the value up to
#   the colon). If neither is set, it is assumed that the audio-server runs 
#   on the \fIlocalhost\fR.
#   
#   RETURN VALUES
#   In case of success, the audio-server type is returned. It can take the
#   value of \fISnack\fR, \fIHP\fR or \fINCD\fR, depending also on the set of supported
#   devices audioPlay has been configured for via the Makefile.
#
#   CAUTIONS
#   Use of audio on NCD terminals requires that the sound files are on a 
#   directory which the terminal can NFS-mount. On top of that, the NCD
#   software must on the HP-UX box be installed under a particular directory,
#   as defined in the module's Makefile. If this is not the case, there will
#   be no audio played on the NCD terminal.
#
#   Both audioServer and HP's Aserver can be running on a different host than 
#   where the audioPlay client runs. But due to the proprietary nature of HP's
#   audio software library, the use of HP's Aserver can only work if the
#   audioPlay client too is running on HP-UX. For NCD X-terminals, the NCD 
#   client software must be installed on the local machine - and this is only 
#   available for HP-UX; the AUDIO or DISPLAY variable must of course in this
#   case point to the NCD X-terminal (on top of the other requirements listed
#   above).
#
#   EXAMPLES
#   You have the choice between giving the fully qualified commandname (i.e.
#   including its namespace) or importing it into your current namespace.
#   For that reason, the following 2 examples are equivalent:
#   ...
#   tcl>package require Audio
#   ...
#   tcl>namespace import audio::*
#   tcl>audioPlay -volume 92 $env(VLTROOT)/sounds/flush.au
#   Snack
#   tcl>
#
#   ...
#   tcl>package require Audio
#   ...
#   tcl>audio::audioPlay -volume 92 $env(VLTROOT)/sounds/flush.au
#   Snack
#   tcl>
#
#   SEE ALSO
#   audioServer(1), audioCtrl(1),
#   \fIThe Snack Sound Toolkit\fR (http://www.speech.kth.se/snack/)
#
#------------------------------------------------------------------------
# Remark: this file is used both as a proc for a library and as the source
# for an executable; see the few lines of code at the end of this file.
#
# Ensure the namespace ::audio exists
audio::_initNamespace

proc ::audio::audioPlay {args} {
    variable cfg
    set vol {}
    for {set i 0} {$i < [llength $args]} {incr i} {
	# remark: using "lvarpop" would imply a need for TclX
	switch -glob -- [lindex $args $i] {
	    -vol* {
		set vol [lindex $args [incr i]]
		if {[string length $vol] == 0} break
		if {$vol < 0 || $vol > 100} {error "volume out of range"}
	    }
	    -port {
		set port [lindex $args [incr i]]
	    }
	    default {break}
	}
    }

    # Options should be defined and there should be just 1 argument left
    if {$i != [expr {[llength $args] - 1}]} {
	error "wrong # args: should be \"[lindex [info level 0] 0] ?-vol <volume>? soundFile\""
    } else {
	set soundFile [lindex $args $i]
    }

    if {![file exists $soundFile]} {
	error "audio-file $soundFile does not exist"
    }
	
    set serverHost [_serverHost]

    if {![info exists port]} {set port $::audio::cfg(port)}
    if {![info exists cfg($serverHost,type)]} {
        if {[catch {audioVerify} msg]} {
            error $msg
	}
    }
    if {[catch {::audio::_playOn$cfg($serverHost,type)\
		    $soundFile $serverHost $port $vol} msg]} {
	error $msg
    } else {
	return $cfg($serverHost,type)
    }
}

proc ::audio::_playOnSnack {soundFile serverHost port vol} {
    variable cfg
    variable refCnt

    # Open the sound file as a binary file
    if {[catch {open $soundFile} fd]} {
	error $fd
    }
    fconfigure $fd -translation binary -encoding binary
    
    # Check if there is an snack audioServer running on that host,
    # i.e. try to open binary socket connection to aserver.tcl
    if {[string length $vol] > 0} {
	if {[catch {_sendCmd setv} sock]} {
	    close $fd
	    error "no Snack audio server listening (anymore) at $serverHost/$port"
	} else {
	    puts $sock $vol
	    close $sock
	    set cmd fply;	# forced play
	}
    } else {
	set cmd play
    }
    # Notify the server that a play operation is due
    if {[catch {_sendCmd $cmd} sock]} {
	close $fd
	error "no Snack audio server listening (anymore) at $serverHost/$port"
    } else {
	# Get handle for this operation from the server.
	# Can be used to stop the operation later. (Not shown here.)
	set handle [gets $sock]

	# rest is all binary
	fconfigure $sock -translation binary -blocking 1 -encoding binary

	incr refCnt
	# Write audio data as long as the server can accept more
	fileevent $sock writable "set data \[read $fd 20000\];\
                                  puts -nonewline $sock \$data;\
                                  if {\[eof $fd\]} {\
                                      close $fd; close $sock;\
                                      incr audio::refCnt -1\
                                  }"
	return Snack
    }
}

proc ::audio::_playOnHP {soundFile serverHost port vol} {
    variable cfg
    variable refCnt
    variable currentRate

    if {![string equal $::tcl_platform(os) HP-UX]} {
	# No alternatives for Sun/Linux; actually for Solaris there is
	# /usr/bin/audioplay, but let us conveniently forget about this ;-)
	error "can only send sounds to HP devices from HP-UX"
    }
    switch -- [info command snack::sound] {} {
	set start [clock seconds]
	# we don't use the graphic part of 'snack', so just load 'sound'
	if {[catch {package require sound} msg]} {
	    error "snack not found"
	}
	set end [clock seconds]
	if {[expr {$end - $start}] > 5} {
	    error "HP audio HW problem or Aserver not active"
	}
    }
    set outputs [snack::mixer outputs]
    if {[llength $outputs] == 0 && [llength [snack::mixer inputs]] == 0} {
	error "HP audio HW problem or Aserver not active"
    }	    
    foreach output $outputs {
	# add all the possible outputs to the default ones
	snack::mixer output $output
    }

    set currentVol [audioVolume]
    if {[string length $vol] > 0} {
	if {$vol == 0} {
	    # cheat here, as gain 0 on HP means muting for us
	    ###set vol 1
	}
	audioVolume $vol
    }
    
    set snd [snack::sound -debug 0 -file $soundFile]

    # Snack seems to have problems to mix sounds with different sampling rates.
    # So if this sound has a different rate than the currently playing one,
    # wait until the latter is finished
    set rate [$snd cget -rate]
    if {$refCnt > 0 && $rate != $currentRate } {
	audioFlush 
    }
    set currentRate $rate
    incr refCnt
    # Remark that snack seems reports too early that it is ready, leading to
    # "Audio Error: Function Unimplemented by Aserver" if the application
    # exits immediately after. So take extra time to decrease reference count
    # (tested empirically on Envizex).
    set callback "after 800 incr audio::refCnt -1; \
                  if {!$currentVol} {audio::audioVolume 0}; \
                  $snd destroy"
    if {[catch {$snd play -blocking false -command $callback} msg]} {
	incr refCnt -1
	$snd destroy
	error "HP audio HW problem"
    } else {
	return HP
    }
}

proc ::audio::_playOnNCD {soundFile serverHost port vol} {
    if {![string equal $::tcl_platform(os) HP-UX]} {
	error "can only send sounds to NCD devices from HP-UX"
    }
    set ncdPath $::audio::cfg(ncdPath)
    if {![file exists $ncdPath/xpsh]} {
	error "NCD client program not found"
    }
    if {[string length $vol] > 0} {
	set state [catch {exec $ncdPath/xpsh aplay -f $soundFile -p -v $vol} msg]
    } else {
	set state [catch {exec $ncdPath/xpsh aplay -f $soundFile -p} msg]
    }
    if {$state} {
	error "audio HW/server problem on NCD (check if sound directory can be NFS mounted)"
    } else {
	return NCD
    }
}

# If this file is sourced as a utility, pick up params and play the file.
# Be sure it was not sourced at level 0 by another script (that may also have
# unset the argv0 variable before we get here).
if {[info level] == 0 && [info exists argv0] && [string equal [info script] $argv0]} {
    if {[catch {eval audio::audioPlay $argv} msg]} {
	puts stderr $msg
	exit 1
    } else {
	audio::audioFlush
    }
    exit 0
}
