#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: bobBaseObject.tcl,v 1.145 2005/10/27 09:07:06 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# mbec      09/04/1997  created
# eallaert  14/04/1997  revised manpage etc
# mbec      16/04/1997  manpage & vltstandard - last touch
# eallaert  02/05/1997  only try to talk to OH if properly configured
# mbec	    07/05/1997  pixmap are private common defined in this class
# mbec	    09/05/1997  added 'updateRawData' method
# mbec      06/06/1997  don't wait for replies on status reporting 
# mbec      06/06/1997  new type templateClient - used in sendStatus (tplsrv)
# mbec      10/07/1997  header update
# eallaert  2000-12-11  don't send status to OH if $gvar(tplEvents) is 0
# eallaert  2001-06-05  ... except when a "force" flag is set (for PAFReady)
# eallaert  2001-11-28  delete also OB-log when resetStatus method is called
# eallaert  2003-01-17  separation of "execution status" and "flow ctrl flags"
#                       (needed for implementation of SPR 20010762)
#

#************************************************************************
#   NAME
#   BobBaseObject - base class for Template and Observation Blocks object
# 
#   SYNOPSIS
#   BobBaseObject <obj_name>|#auto <type> <rawData> <canvas>\
#                        ?-<publicVar> <value>?
# 
#   DESCRIPTION
#   This class inherits from BobVisualPCTree and defines a common base for TPL
#   and OB objects. This includes login window functionality, the stepping
#   mode (NORMAL / BREAK / SKIP), handles selection and the initialisation
#   of data array.
#
#   METHODS
#   constructor: see [incr Tcl]
#   destructor: destroy the object and unmark if required; see [incr Tcl]
#   configure: see [incr Tcl]
#
#   disableFlowCtrlChange: effectively disable the use of the "flowCtrlChange"
#       method
#   enableFlowCtrlChange: effectively enable the use of the "flowCtrlChange"
#       method
#   draw <ulx> <uly>: draw the object, add the step mode icon, add the 
#       bindings to 'flowCtrlChange' and the mark which will highlight the 
#	object and display its log; return the proper <uly> for a next similar
#       object. 
#   getObdSource: return obd source (one of OH file or TemplateClient)
#   getRawData: returns the rawData used for creation
#   paste: first check for type mismatch etc... extende by subclass
#   populate: creates the sub-tree objects after the data array has been 
#       created - virtual here
#   mark: select this object
#   setRawData <rd>: fills the raw data bit
#   toggle_mark: toggle marked object
#   flowCtrlChange: allows to set the current flowCtrl value (NORMAL / BREAK /
#        SKIP);	update the canvas at the same time
#   unmark: unmark this object
#         
#   CLASS PROCEDURES
#   marked: returns currently marked object
#   markedIsOB: returns 1 if currently marked object is an OB
#   resetStatus: reset the basic configuration for all objects on the canvas:
#       status = idle
#       PC = -1
#       clear templates logs
#   sendStatus <cmd> <obds> <str> ?<force>? : send the SCRIPT command to the
#       schedule process (OH) with <cmd> and <str> as arguments, provides the
#       source of the OBD is OH (i.e. <obds> is set to "OH"). The <force> flag
#       is for use in combination with ::bob::internal::config(tplEvents): if 
#       the latter is false, there are no "events" sent to OH, unless the
#       <force> flag is set; by default <force> is false.
#
#   VARIABLES
#   names: defines array of key for data storage
#   status: define the cstatus of the object (idle / running / paused / 
#         aborted / fault)
#   flowCtrl: define the stepping through mode (NORMAL / BREAK /SKIP)
#   type: defines type of the object for data storage
#	
#   rawData (protected)
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#------------------------------------------------------------------------
#

#=============================================================================
#=============================================================================
itcl::class BobBaseObject {
    inherit BobVisualPCTree

    constructor {t rd obds cwin args} {
	BobVisualPCTree::constructor $cwin
    }  {}
    
    destructor {}

    public method disableFlowCtrlChange {} {};
    public method enableFlowCtrlChange {} {};
    public method draw {ulx uly} {}
    public method getObdSource {} {return $obdSource}
    public method getRawData {} {}
    public method mark {} {}
    public method paste { obj where } {}
    public method populate {} {};		# virtual here
    public method setRawData { rd } {}
    public method toggle_mark {} {}
    public method flowCtrlChange {} {}
    public method unmark {} {}

    public proc marked {} {}
    public proc markedIsOB {} {}
    public proc resetStatus {} {}

    protected proc sendStatus {args} {}

    private method setStateIcon {args} {}

    public variable names;	# name is the ordered list with param names
    # (used only for CAT class)
    public variable status	"idle"
    public variable flowCtrl	"NORMAL"
    public variable type {}
    
    private variable currentStatus	"idle"
    private variable stateIcon		"img_NORMAL"
    # creation data used to fill the array; kept if not unfolded automatically
    # and also for the save - copy/paste etc..
    protected variable rawData	{};
    protected variable rdFormat {%-30s "%s"};	# to reconstruct rawData string

    # either OH / file or templateClient; controls sendStatus as follows:
    # OH  		=>	msg_sendcmd
    # file 		=>	nothing
    # templateClient	=>	msg_sendreply
    private variable obdSource;
    
    # flags if flow control change is allowed (typically not when running/done)
    private variable flowCtrlMayChange 1;
    private common markedObj	"";	# only one marked object at a time now

    private common img_aborted	[image create pixmap img_aborted -file [bobFindVlt bitmaps/bobAborted.xpm]]
    private common img_done	[image create pixmap img_done -file [bobFindVlt bitmaps/bobDone.xpm]]
    private common img_fault	[image create pixmap img_fault -file [bobFindVlt bitmaps/bobError.xpm]]
    private common img_paused	[image create pixmap img_paused -file [bobFindVlt bitmaps/bobPaused.xpm]]
    private common img_running	[image create pixmap img_running -file [bobFindVlt bitmaps/bobRunning.xpm]]
    private common img_BREAK	[image create pixmap img_BREAK -file [bobFindVlt bitmaps/bobBreak.xpm]]
    private common img_NORMAL	[image create pixmap img_NORMAL -file [bobFindVlt bitmaps/bobNormal.xpm]]
    private common img_SKIPPED	[image create pixmap img_SKIPPED -file [bobFindVlt bitmaps/bobSkip.xpm]]

    # class initializer : check for common size and update if required
    foreach img {img_SKIPPED img_BREAK img_NORMAL img_running img_done\
	    img_aborted img_fault img_paused }  {
	
	if {[set x [image height $img]] > $icon_h} {set icon_h $x}
	if {[set x [image width $img]] > $icon_w} {set icon_w $x}
    }
}

#=============================================================================
#=============================================================================

# ------------------------------------------------------------------
#                        CONSTRUCTOR
# ------------------------------------------------------------------
itcl::body BobBaseObject::constructor {t rd obds cwin args} {
    set rawData $rd
    set type $t
    set obdSource $obds
    eval configure $args
}

# ------------------------------------------------------------------
#                        DESTRUCTOR
# ------------------------------------------------------------------
itcl::body BobBaseObject::destructor {} {
    # remove if marked
    switch -- $markedObj $this {
	set markedObj {}
    }
}

# ------------------------------------------------------------------
#                        PUBLIC METHOD disableFlowCtrlChange
# ------------------------------------------------------------------
itcl::body BobBaseObject::disableFlowCtrlChange {} {
    set flowCtrlMayChange 0
}

# ------------------------------------------------------------------
#                        PUBLIC METHOD draw
# ------------------------------------------------------------------
itcl::body BobBaseObject::draw {ulx uly} {
    
    set moretags [$this familytag]
    append moretags " childtag-$this"
    
    if {[catch {$canvas create image $ulx [expr {$uly+$icon_h/2}] \
	    -image $stateIcon -anchor e \
	    -tags [eval list $this $this-state $moretags]} msg]} {
	# Debugging for "bad screen distance" - SPR 20000553
	seq_errResetStack
	seq_errAdd "Tcl error-msg: $msg" bob
	seq_errAdd "Tcl/Tk error in BobBaseObject::draw,\
		ulx = >$ulx<, uly = >$uly<" bob
	seq_errCloseStack
	return $uly
    }
    $canvas create rectangle [expr {$ulx+$icon_w}] \
	    $uly 1500 [expr {$uly+$icon_h}] \
	    -outline "" -fill "" \
	    -tags [eval list $this hilite $this-hilite $moretags]
    $canvas bind $this-text <ButtonPress-1> "$this toggle_mark"
    $canvas bind $this <ButtonPress-3> "$this flowCtrlChange"
    
    set uly [BobVisualTree::draw $ulx $uly]
    $canvas itemconfigure $markedObj-hilite -fill LightBlue
    $canvas raise $this-state
    return $uly
}

# ------------------------------------------------------------------
#                        PUBLIC METHOD enableFlowCtrlChange
# ------------------------------------------------------------------
itcl::body BobBaseObject::enableFlowCtrlChange {} {
    set flowCtrlMayChange 1
}

# ------------------------------------------------------------------
#                        PUBLIC METHOD flowCtrlChange
# ------------------------------------------------------------------
# Forbid a 'flowCtrlChange' if not idle. However, we want to be able to 
# skip a TPL (not a OB !!!) that has been been paused before starting ...
itcl::body BobBaseObject::flowCtrlChange {} {
    if { $flowCtrlMayChange } {
	switch -- $status idle  {
	    switch -- $flowCtrl {
		NORMAL {
		    configure -flowCtrl SKIPPED
		}
		SKIPPED {
		    configure -flowCtrl BREAK
		}
		BREAK {
		    configure -flowCtrl NORMAL
		}
	    }
	} paused {
	    if {[$this isa BobTPL]} {
		switch -- $flowCtrl {
		    SKIPPED {
			set flowCtrl BREAK
			setStateIcon $status
		    }
		    BREAK - NORMAL {
			configure -flowCtrl SKIPPED
		    }
		}
	    }
	}
    }
}

# ------------------------------------------------------------------
#                        PUBLIC METHOD getRawData
# ------------------------------------------------------------------
itcl::body BobBaseObject::getRawData {} {
    return $rawData
}

# ------------------------------------------------------------------
#                        PUBLIC METHOD mark
# ------------------------------------------------------------------
itcl::body BobBaseObject::mark {} {
    $canvas itemconfigure $markedObj-hilite -fill ""
    set markedObj $this
    $canvas itemconfigure $this-hilite -fill LightBlue
}

#---------------------------------------------------------------------------
#                        PUBLIC METHOD paste
#---------------------------------------------------------------------------
itcl::body BobBaseObject::paste {obj where} {
    # check it's not void
    switch -- [itcl::find objects $obj] {} {
	bobDialog "NO OBJECT ready for pasting : use Cut or Copy first !" \
		application
	return -code return
    }
    # check it's same class
    if {![$obj isa [$this info class]]} {
	bobDialog "type mismatch !\nplease select a object of the same class\
		to paste below\npaste buffer: $obj\nselected object: $this" \
		application
	return -code return
    }
}
# ------------------------------------------------------------------
#                        PUBLIC METHOD setRawData
# ------------------------------------------------------------------
# this method is meant for raw data update (cut/paste operation)
# this method is extended by bobOB and bobTPL class which are actually
# doing the job
itcl::body BobBaseObject::setRawData {rd} {
    #puts "=============================/n/nrawData for $this is $rd"
    set rawData $rd
}

# ------------------------------------------------------------------
#                        PUBLIC METHOD toggle_mark
# ------------------------------------------------------------------
itcl::body BobBaseObject::toggle_mark {} {
    switch -- $markedObj $this {
	unmark
    } default {
	mark
    }
}

# ------------------------------------------------------------------
#                        PUBLIC METHOD unmark
# ------------------------------------------------------------------
itcl::body BobBaseObject::unmark {} {
    set markedObj {}
    $canvas itemconfigure $this-hilite -fill {}
}



# ------------------------------------------------------------------
#                        PUBLIC VAR flowCtrl
# ------------------------------------------------------------------
itcl::configbody BobBaseObject::flowCtrl {
    if {[string equal $status idle]} {
	setStateIcon $flowCtrl
    } else {
	error "cannot change flow control if not idle"
    }
}

# ------------------------------------------------------------------
#                        PUBLIC VAR status
# ------------------------------------------------------------------
#assumes you give it a good value
#we loose the step step as we go through
itcl::configbody BobBaseObject::status {
    switch -- $status idle {
	set abort 0
	set mustRepeat 0
	set pause 0
	set flowCtrlMayChange 1
	setStateIcon $flowCtrl
    } default {
	# if aborted or error - no way to change the status
	switch -- $currentStatus aborted - fault {
	    error "bug: cannot change status from $currentStatus to $status\
                   (try the \"Reset status\" button)"
	}
	setStateIcon $status
    }
    set currentStatus $status
    #puts "config $this status = $status  flowCtrl = $flowCtrl"
}

# ------------------------------------------------------------------
#                        CLASS PROCEDURE marked
# ------------------------------------------------------------------
itcl::body BobBaseObject::marked {} {
    if {[info exists markedObj]} {
        return $markedObj
    }
    return {}
}
# ------------------------------------------------------------------
#                        CLASS PROCEDURE markedIsOB
# ------------------------------------------------------------------
itcl::body BobBaseObject::markedIsOB {} {
    if {[string length $markedObj] != 0 && [$markedObj isa BobOB]} {
	return 1
    } else {
	return 0
    }
}
# ------------------------------------------------------------------
#                        CLASS PROCEDURE resetStatus 
# ------------------------------------------------------------------
itcl::body BobBaseObject::resetStatus {} {
    if {[string length [root]] == 0} return
    foreach OBobj [[root] contents] {
	# Find out the "stepping state" of this OB and its templates
	set OBflowCtrl [$OBobj cget -flowCtrl]
	# reset any termination state to NORMAL
	switch -- $OBflowCtrl SKIPPED - BREAK {} default {
	    set OBflowCtrl NORMAL
	}
	set TPLflowCtrls {}
	foreach TPLobj [$OBobj contents] {
	    set flowCtrl [$TPLobj cget -flowCtrl]
	    switch -- $flowCtrl SKIPPED - BREAK {} default {
		set flowCtrl NORMAL
	    }
	    lappend TPLflowCtrls $flowCtrl
	}

	# reset the OB
	$OBobj configure -status idle
	$OBobj resetPC
	$OBobj forceFinish {}

	# Rebuild all templates from scratch (this is why a "resetStatus" of
	# the individual templates is not needed).
	$OBobj rePopulate
	$OBobj deleteLog

	# Restore the original stepping states of the OB and its templates.
	$OBobj configure -flowCtrl $OBflowCtrl
	foreach TPLobj [$OBobj contents] {
	    $TPLobj configure -flowCtrl [lvarpop TPLflowCtrls]
	}
    }
    seq_logFitsEvent -ufo "user requested RESET of OB's execution status"
}
# ------------------------------------------------------------------
#                        CLASS PROCEDURE sendStatus
# ------------------------------------------------------------------
# ---------------------------------------------------------------------
# The ICD tells the "status" should be signalled as events, which means
# that BOB is not really interested in replies.  The point is that OT
# may be busy (for minutes!!) on other stuff, so it will not react
# immediately on these events (i.e. should not wait for the final reply 
# to the CCS message it has sent).
# ---------------------------------------------------------------------
itcl::body BobBaseObject::sendStatus {cmd obds str {force 0}} {

    switch -- $obds OH {
    	if {[string length $::bob::internal::config(bobEnvNameOH)] == 0 || \
		[string length [lindex $::bob::internal::config(bobProcNameOH) 0]] == 0 || \
		(!$::bob::internal::config(tplEvents) && !$force) } {
	    return 0
    	}

	# needed in case some of the config vars refer to environment vars:
	global env
    	if {[catch {seq_msgSendCommand -nocheck -noreply \
		[subst $::bob::internal::config(bobEnvNameOH)] \
		[subst [lindex $::bob::internal::config(bobProcNameOH) 0]] EVENT "$cmd $str"}]} {
	    bobDialog "Could not communicate with OH.\n\
		    Check the Environment settings (under Configure menu)\n\
		    and the CCS error-log." application
	    seq_errDisplay -suspend
	    return 1
    	}
	return 0
    }
}    

# ------------------------------------------------------------------
#                        PRIVATE METHOD setStateIcon
# ------------------------------------------------------------------
itcl::body BobBaseObject::setStateIcon {state} {
    set stateIcon img_$state
    $canvas itemconfigure $this-state -image $stateIcon
}

#
# ___oOo___






