#******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: bobOB.tcl,v 1.145 2005/10/27 09:07:12 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# eallaert  1996-11-14  created
# eallaert  1997-02-18  show error stack trace for failing template execution
# mbec      1997-04-09  modified
# eallaert  1997-04-14  revised manpage, added params to OBSinfo
# mbec      1997-04016  manpage & vltstandard - last touch
# mbec      1997-05-09  added 'updateRawData' method
# mbec      1997-05-13  added editRawData method
# mbec      1997-07-10  header update
# eallaert  1997-09-25  some clean-up, OBS.DID added
# eallaert  1998-04-07  mustRepeat added
# eallaert  1999-08-03  added sending of OBS.START to OS (SPR 990308)
# eallaert  1999-11-05  do not use "status" as method argument (shadowing
#                       problem in itcl3.0 with respect to object vars)
# eallaert  2000-04-14  prepare for parallel templates 
# eallaert  2001-04-24  added FITS ops-logs (SPR 20010098)
# eallaert  2001-08-14  added forceFinish handling
# eallaert  2002-04-24  added "re" arg to "populate"
#                       add log to OB if pausing/continuing execution
#                       don't recalculate OBS(START) after pause (SPR 20020301)
# eallaert  2003-01-15  identify OB in template log when starte (SPR 20030520)

#

#************************************************************************
#   NAME
#   BobOB - base class for Observation blocks objects
# 
#   SYNOPSIS
#   BobOB <obj_name>|#auto <rawData> <obdsource> <canvas> \
#	?-<publicVar> <value>? ...
# 
#   DESCRIPTION
#   BobOb defines the class for the Observation Blocks (OBs).  It inherits
#   BobBaseObject.
#   The usual [incr Tcl] rule for giving configuration parameters together
#   when invoking any method applies.
#   
#   METHODS
#   constructor: creates the object and store in array the OBS elements
#                update label consequently for this OB
#                NB: the templates are not generated at construction time.
#   destructor: see [incr Tcl]
#   configure: see [incr Tcl]
#
#   abort ?<msg>?: request abort (also process it if execution is paused); if
#     no <msg> is given, prompt for one via a dialog window.
#   execute: handles execution of the OB
#   forceFinish ?-status <status>?: when the currently executing template
#     finishes, skip the remaining templates, and return <status> to OH
#   getObsKey <key>: return $OBS(<key>)
#   mark: extended to display log of all associated templates
#   mustRepeat ?<msg>?: request MUSTREPEAT (implies an abort); if no <msg>
#     is given, prompt for one via a dialog window.
#   obsId: return $OBS(ID)
#   paste <obj> <there> : paste <obj> <there> relative to this
#   pause: request pause
#   populate: creates all Template child objects from raw data
#   printLog: prints log of OB (ie. of all associated templates)
#   sendStatus <status> <args>: send SCRIPT to schedule OH with 
#         ObsBlockStatus and arguments
#   toggle_fold: extended to populate (cf. autoCreate variable)
#   toggle_foldAll: extended to populate (cf. autoCreate variable)
#   updateRawData : set raw data from arrays value
#
#   CLASS PROCEDURES
#
#   VARIABLES
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#   ICD between the VLT Control SW and the Observation Handling Subsystem
#   (VLT-ICD-ESO-17240-19200)
#------------------------------------------------------------------------
#

#=============================================================================
#=============================================================================
itcl::class BobOB {
    inherit BobBaseObject BobLogger
    
    constructor {rd obds cwin stwin args} {
	BobBaseObject::constructor OBS $rd $obds $cwin
	BobLogger::constructor $stwin
    } {}

    public method abort {args} {}
    public method execute { } {}
    public method forceFinish {args} {}
    public method getObsKey {key} {}
    public method getResult {args} {};	# get result of previous template
    public method mark {} {}
    public method mustRepeat {args} {}
    public method obsId {} {}
    public method paste {obj where} {}
    public method pause {args} {}
    public method populate {args} {};	# public to let OB populate later
    public method printLog {} {}
    public method rePopulate {args} {}
    public method setResult {args} {};	# store for use in later template
    public method toggle_fold {} {};	# extends to add the populate method
    public method toggle_foldAll {} {};	# extends to add the populate method
    public method updateLabel {args} {} ;
    public method updateRawData {} {} ;

    private method doCallBack {args} {};
    private method editRawData {args} {};# edit raw data in given scrolled frame
    private method getEditedData {args} {}
    private method _sendStatus {status args} {}
    private proc parseTPL {block} {};	# parse raw data and create
					# corresponding array
    private variable OBS;		# array with <OBS.> info from template
    private variable OBSinfo {};
    private variable populated 0;	# to know when it's been done
    private variable abortMotive {};	# text collected with the prompt dialog
					# after a confirmed abort request
    private variable results;		# array to store results of templates
    private variable forcedFinishStatus {};	# special termination status

    private common nsc {not set};	# namespace of object; flag unitialized
    private common startTime {};	# clock-time when OB was started
}

#=============================================================================
#=============================================================================

# ------------------------------------------------------------------
#                        CONSTRUCTOR
# ------------------------------------------------------------------
# bobStoreInArray is done in this context
itcl::body BobOB::constructor {rd obds cwin stwin args} {
    if {[catch {bobStoreInArray $rawData $type} msg]} {
	bobDialog "There is something wrong with the parameters of this OB:\n\
		--------------------------------------------------------\n\
		$msg\n\
		--------------------------------------------------------\n\
		I will proceed, but you may find this OB crippled.\n\
		I suggest to check this, and eventually to reload this OB." \
		application
    }
    switch -- $nsc {not set} {
	set nsc [namespace current]
	switch -- $nsc :: {set nsc {}}
    }

    set OBS(DID) $::bob::internal::bobObsDid
    # values in param-value pairs should contain quotes !!!
    foreach el [lsort [array names OBS]] {
	append OBSinfo " OBS.${el} \"$OBS($el)\""
    }

    eval configure $args
    updateLabel
}

# ------------------------------------------------------------------
#                        PUBLIC METHOD abort
# ------------------------------------------------------------------
# The msg argument is there basically for non-interactive use (bobWish-script)
itcl::body BobOB::abort { {msg {}} } {
    global gvar
    
    if {$mustRepeat} {
	set promptMotive "Please specify motive to MUSTREPEAT        "
	set statusToSend MUSTREPEAT
	set childAction mustRepeat
	set errorMsg "Cannot request to repeat this OB."
    } else {
	set promptMotive "Please specify motive to ABORT             "
	set statusToSend ABORTED
	set childAction abort
	set errorMsg "Cannot ABORT this OB."
    }
    # to filter any uncompatible event:
    if {![string equal $status done] && ![string equal $status fault]} {
	if {[string length $msg] != 0} {
	    set ans [list 1 $msg]
	} else {
	    # first check if you're sure
	    set ans [bobPromptDialog $promptMotive application 0 info]
	}
	if {[lindex $ans 0]==1} {
	    set abortMotive [lindex $ans 1]
	    # if OB is paused I handle abort/mustrepeat myself; same if this
	    # abort action is seen just after the last template terminated.
	    switch -- $status paused {
		if {$::bob::internal::config(bobAutoUnfold) && \
			[string equal [$activeChild cget -fold_state] unfolded]} {
		    # I use catch in case the activeChild is already undefined
		    catch {$activeChild foldAll "folded"}
		}
		# disable all following templates to prevent flow control change
		while {[string length $activeChild] != 0} {
		    $activeChild disableFlowCtrlChange
		    setActiveChild [getNextChild]
		}
	    } default {
		if {![catch {$activeChild $childAction}]} {
		    # This means there are still template(s) executing
		    set abort 1
		    return
		}
		# getting here means the ABORT has been caught after the
		# execution of the last active child; so there's no 
		# activeChild anymore; send the status directly from here
	    }
	    configure -status aborted
	    _sendStatus $statusToSend "motive: $abortMotive"
	    set stopTime [clock seconds]
	    mark;	# highlight OB & display entire log
	    logLog "OB aborted after [expr {$stopTime-$startTime}] seconds at\
		    [clock format $stopTime -format %Y-%m-%dT%H:%M:%S -gmt 1]" \
		    black 0 LightBlue
	    ::bob::internal::resetButtons
	    seq_logFitsEvent -ufo "user requested $statusToSend"
	    seq_logFitsAction OBS OBS EXEC ABORT
	    doCallBack $::bob::internal::config(bobCallBack)
	    logToDisk "OB LOG: [cget -label]"
	}
    } else {
	bobDialog "$errorMsg\n\
		Such request is incompatible with the current OB status." \
		application 10000
    }
}
# ------------------------------------------------------------------
#                        PUBLIC METHOD execute
# ------------------------------------------------------------------

# returns 0 if done/skipped
# returns 1 if fault/aborted
# returns 2 if paused
itcl::body BobOB::execute { } { 
    global gvar
    
    populate;	# populate now -if not done before
    mark;	# highlight current object

    # Check if there's a skip
    switch -- $flowCtrl SKIPPED {
	# Fold the OB back if so required
	if {$::bob::internal::config(bobAutoUnfold) && [string equal $fold_state unfolded]} {
	    foldAll folded
	}
	return 0
    } 

    # Verify if OK to start, and take actions accordingly
    switch -- $status aborted - fault - done {
	bobDialog "Cannot start this OB!\n\
		Its state is not idle.\n\
		Try the \"Reset status\" action button." \
		application 10000
	::bob::internal::resetButtons
	return 1

    } paused {
	# make sure not to reset the log window nor OBS(START)
	logLog "OB continued at [clock format [clock seconds] \
		-format %Y-%m-%dT%H:%M:%S -gmt 1]" black 0 LightBlue
	logLog ""
	_sendStatus CONTINUED
	seq_logFitsAction OBS OBS EXEC RESUME
	set pause 0;	# to prevent any further flow control change

    } idle {
	deleteLog;	# start from a clean log-box
	set OBS(START) [clock format [set startTime [clock seconds]] \
		-format %Y-%m-%dT%H:%M:%S -gmt 1]
	logLog "OB started at $OBS(START)" black 0 LightBlue
	logLog "(OBS.NAME: $OBS(NAME) // OBS.ID: $OBS(ID))"
	logLog ""
	_sendStatus STARTED
	seq_logFitsAction OBS OBS EXEC START
	# catch errors, e.g. if OBS(ID) would be non-numeric
	catch {seq_logFitsParRecord -int OBS OBS "" ID $OBS(ID)}
	seq_logFitsParRecord -string OBS OBS "" NAME $OBS(NAME)
	resetPC
	set activeChild [getNextChild]
	# Show the OB's templates 
	if {$::bob::internal::config(bobAutoUnfold)==1 && [string equal $fold_state folded]} {
	    toggle_fold
	}
	
	# If there is BREAK programmed, we "activate" it after starting the OB
	if {[string equal $flowCtrl BREAK]} {
	    _sendStatus PAUSED
	    seq_logFitsAction OBS OBS EXEC PAUSE
	    configure -status paused
	    set pause 1
	    ::bob::internal::resetButtons 0 continue 1 1 1 0
	    return 2
	}

    } default {
	bobDialog "Seems like a bug: starting an already running OB ?!?!\n\n\
		Please report this back to the software engineers."
		application 10000
	::bob::internal::resetButtons
	return 1
    }
    
    # it could be you folded everything while paused, so unfold again 
    if {$::bob::internal::config(bobAutoUnfold)==1 && [string equal $fold_state folded]} {
	toggle_fold
    }

    ::bob::internal::resetButtons 0 pause 1 1 1 0; # configure the ctrl buttons
    configure -status running;	# now we're running
    
    set finishStatus TERMINATED;	# default
    # loop until an error is encountered, or abort command was given.  This is
    # actually reflected in the "status"-variable, by processing it here.
    while {[string length $activeChild] != 0} {
	# Build the OBS-category information the Tpl has to send to OS
	set tplNr [expr {[getPC] + 1}]
	$activeChild mark
	set msg [$activeChild execute \
		"$OBSinfo OBS.START \"$OBS(START)\" OBS.TPLNO \"$tplNr\"" \
		$tplNr stat]
	appendLog [$activeChild getLogTextWithTags]
	if {$stat == -2} {
	    set pause 1;		# template was paused 
	    # do NOT return right now since there could be an ABORT 
	    # pending which I want to detect on top ...
	    # all this is handled later (see ***) with "pause" set to 1
	} elseif {$stat == -1 || $stat == -3} {
	    # template aborted or has detected abort pending
	    set abort 1
	} elseif {$stat == 0} {
	    setActiveChild [getNextChild]
	    if {[string length $forcedFinishStatus] != 0} {
		# disable all following templates to prevent flow control change
		while {[string length $activeChild] != 0} {
		    $activeChild configure -status SKIPPED
		    $activeChild disableFlowCtrlChange;	# disable further manipulation
		    setActiveChild [getNextChild]
		}
		set finishStatus $forcedFinishStatus
	    }
	} elseif {$stat > 0} {
	    configure -status fault
	    _sendStatus "ABORTED" "template error: $msg"
	    seq_logFitsEvent -ufo {template error}
	    seq_logFitsAction OBS OBS EXEC ABORT
	    ::bob::bobPlaySound [bobFindSound $::bob::internal::config(bobErrorSound)]
	    set stopTime [clock seconds];	# get time before pop-up
	    if {$::bob::internal::config(bobUserLevel) == 1 && [seq_errGetStackSize] > 0} {
		seq_errDisplay;	# SPRs 980567 & 20010005
	    }
	    # disable all following templates to prevent flow control change
	    while {[string length $activeChild] != 0} {
		$activeChild disableFlowCtrlChange
		setActiveChild [getNextChild]
	    }
	    bobDialog $msg application
	    mark;	# highlight OB & display entire log
	    logLog "OB aborted after [expr {$stopTime-$startTime}] seconds at\
		    [clock format $stopTime -format %Y-%m-%dT%H:%M:%S -gmt 1]" \
		    black 0 LightBlue
	    ::bob::internal::resetButtons
	    setActiveChild {}
	    doCallBack $::bob::internal::config(bobCallBack)
	    logToDisk "OB LOG: [cget -label]"
	    return 1 
	}
	
	# abort has priority on pause
	if {$abort} {
	    # abort request pending from GUI after execution of TPL
	    if {$stat ==0} {
		# I fold back only if the abort is not in the template but has
		# been caught later
		if {$::bob::internal::config(bobAutoUnfold) && \
			[string equal [$activeChild cget -fold_state] unfolded]} {
		    # use a catch because there could be NO active child at 
		    # the time of this event.
		    catch {$activeChild foldAll "folded"}
		}
	    }
	    
	    # have to disable all following template to prevent flow control change
	    while {[string length $activeChild] != 0} {
		$activeChild disableFlowCtrlChange
		setActiveChild [getNextChild]
	    }
	    configure -status aborted
	    if {$mustRepeat} {
		_sendStatus "MUSTREPEAT" "motive: $abortMotive"
		seq_logFitsEvent -ufo {user requested MUSTREPEAT}
		seq_logFitsAction OBS OBS EXEC ABORT
	    } else {
		_sendStatus "ABORTED" "motive: $abortMotive"
		seq_logFitsEvent -ufo {user requested ABORT}
		seq_logFitsAction OBS OBS EXEC ABORT
	    }
	    set stopTime [clock seconds]
	    ::bob::internal::resetButtons
	    mark;	# highlight OB & display entire log
	    logLog "OB aborted after [expr {$stopTime-$startTime}] seconds at\
		    [clock format $stopTime -format %Y-%m-%dT%H:%M:%S -gmt 1]" \
		    black 0 LightBlue
	    setActiveChild {}
	    doCallBack $::bob::internal::config(bobCallBack)
	    logToDisk "OB LOG: [cget -label]"
	    return 1 
	} elseif {$pause} {
	    # pause request either pending from action button or TPL BREAK flag
	    mark;	# to have logLog not end up in TPL's log
	    logLog "OB paused at [clock format [clock seconds] \
		    -format %Y-%m-%dT%H:%M:%S -gmt 1]" black 0 LightBlue
	    logLog ""
	    if {$stat ==0} {
		# this means the pause comes from GUI after tpl has been done.
		catch {$activeChild pause};	# pause next template (if any)
		##catch {$activeChild mark};	# highlight it
	    }
	    configure -status paused
	    _sendStatus PAUSED 
	    seq_logFitsAction OBS OBS EXEC PAUSE
	    ::bob::internal::resetButtons 0 continue 1 1 1 0
	    return 2 
	}

	# process pending events after execution of a template
	update
    }
    
    # Fold the OB back if so required
    if {$::bob::internal::config(bobAutoUnfold)&& $fold_state=="unfolded"} {
	foldAll folded
    }

    configure -status done ;      # no more templates in this OB
    seq_logFitsAction OBS OBS EXEC STOP
    _sendStatus $finishStatus

    set stopTime [clock seconds]
    # highlight OB & display entire log; 
    # make sure you do this before appending a log to it.
    mark	
    logLog "OB finished ($finishStatus) in [expr {$stopTime-$startTime}] seconds at\
		[clock format $stopTime -format %Y-%m-%dT%H:%M:%S -gmt 1]" \
		black 0 LightBlue
    ::bob::internal::resetButtons
    doCallBack $::bob::internal::config(bobCallBack)
    logToDisk "OB LOG: [cget -label]"
    return 0
}

# ------------------------------------------------------------------
#                        PUBLIC METHOD forceFinish
# ------------------------------------------------------------------
# The list of valid extra termination states should have been picked 
# up from the instrument summary file.
itcl::body BobOB::forceFinish {status} {
    ###if {[lindex $validFinishStates $status] < 0} {
	###error "wrong status: should be one of $validFinishStates"
    ###}
    # a "reset status" should also reset the forcedFinishStatus!
    set forcedFinishStatus $status;
    return 
}

# ------------------------------------------------------------------
#                        PUBLIC METHOD getObsKey
# ------------------------------------------------------------------
itcl::body BobOB::getObsKey {key} {
    return $OBS($key)
}

# ------------------------------------------------------------------
#                        PUBLIC METHOD getResult
# ------------------------------------------------------------------
itcl::body BobOB::getResult {name} {
    return $results($name)
}

# ------------------------------------------------------------------
#                        PUBLIC METHOD mark
# ------------------------------------------------------------------
itcl::body BobOB::mark {} {
    displayLog
    BobBaseObject::mark
}

# ------------------------------------------------------------------
#                        PUBLIC METHOD mustRepeat
# ------------------------------------------------------------------
# Same as abort
itcl::body BobOB::mustRepeat { {msg {} } } {
    set mustRepeat 1
    abort $msg
}
# ------------------------------------------------------------------
#                        PUBLIC METHOD obsId
# ------------------------------------------------------------------
itcl::body BobOB::obsId {} {
    return $OBS(ID)
}

#------------------------------------------------------------------------------	
#                        PUBLIC METHOD paste
#------------------------------------------------------------------------------	
itcl::body BobOB::paste {obj where} {
    global gvar
    BobBaseObject::paste $obj $where
    
    # make a copy of obj
    set copyobj [BobOB #auto [$obj getRawData] [$obj getObdSource] \
	    $gvar(cvOB) $gvar(stTpl)]
    $copyobj populate
    [up] add ${nsc}::$copyobj $where $this
    return ${nsc}::$copyobj
}

# ------------------------------------------------------------------
#                        PUBLIC METHOD pause
# ------------------------------------------------------------------
itcl::body BobOB::pause {} {
    set pause 1
}

# ------------------------------------------------------------------
#                        PUBLIC METHOD populate
# ------------------------------------------------------------------
itcl::body BobOB::populate {{re 0}} {
    global gvar
    
    if {!$populated} {
	busyCursor
	update
	set tplList [parseTPL $rawData]
	# Get first approximation of start-time, so this can be passed as
	# part of the OBSinfo, for the purposes of parallel templates
	set OBS(START) \
	    [clock format [set startTime [clock seconds]] \
	    -format %Y-%m-%dT%H:%M:%S -gmt 1]
	for {set i 0; set nr 1} {$i<[llength $tplList]} {incr i; incr nr} {
	    set obj [BobTPL #auto [lindex $tplList $i] [getObdSource] \
		    $gvar(cvOB) $gvar(stTpl) \
		    "$OBSinfo OBS.START \"$OBS(START)\" OBS.TPLNO \"$nr\"" $nr]
	    add ${nsc}::$obj
	    $obj populate $re
	}
	set populated 1
	normalCursor
    }
}

itcl::body BobOB::rePopulate { } {
    global gvar

    switch -- $fold_state unfolded toggle_fold
    clearChildren
    foreach el [array names OBS] {
	unset OBS($el)
    }
    if {[catch {bobStoreInArray $rawData $type} msg]} {
	bobDialog "There is something wrong with the parameters of this OB:\n\
		--------------------------------------------------------\n\
		$msg\n\
		--------------------------------------------------------\n\
		I will proceed, but you may find this OB crippled.\n\
		I suggest to check this, and eventually to reload this OB." \
		application
    }
    set populated 0
    populate 1;	# flag that this a *re*population
}

# ------------------------------------------------------------------
#                        PUBLIC METHOD printLog
# ------------------------------------------------------------------
itcl::body BobOB::printLog {} {
    set header "OB LOG: [cget -label]"
    set len [string length $header]
    append header \n
    for {set i 0} {$i < $len} {incr i} {
	append header -
    }
    append header \n\n
    BobLogger::printLog $header
}

# ------------------------------------------------------------------
#                        PUBLIC PROCEDURE setResult
# ------------------------------------------------------------------
itcl::body BobOB::setResult {name value} {
    set results($name) $value
}

# ------------------------------------------------------------------
#                        PUBLIC METHOD toggle_fold
# ------------------------------------------------------------------
itcl::body BobOB::toggle_fold { } {
    # populate now -if not done before
    populate
    BobVisualTree::toggle_fold
}

# ------------------------------------------------------------------
#                        PUBLIC METHOD toggle_foldAll
# ------------------------------------------------------------------
# populate now -if not done before
itcl::body BobOB::toggle_foldAll { } {
    populate
    BobVisualTree::toggle_foldAll
}

# ------------------------------------------------------------------
#                        PUBLIC METHOD updateLabel
# ------------------------------------------------------------------
itcl::body BobOB::updateLabel {} {
    set name {}
    catch {set name $OBS(PI-COI.NAME)}
    if {[catch {configure -label "$OBS(NAME) -- $OBS(PROG.ID) -- $name"}]} {
	configure -label "OB with problems - pls correct and reload"
    }
}
# ------------------------------------------------------------------
#                        PUBLIC METHOD updateRawData
# ------------------------------------------------------------------
# first get the OBS stuff; remark that ID has to be the first line !!!
# Also, do not include the OBS.START keyword
itcl::body BobOB::updateRawData {} {
    set ob "# OB edited by BOB\n[format $rdFormat OBS.ID $OBS(ID)]\n"
    foreach key [lsort [array names OBS]] {
	switch -- $key ID {} START {} default {
	    append ob [format $rdFormat OBS.$key $OBS($key)]\n
	}
    }
    
    append ob "\n\n\n"
    # now handle TPL, so that it correspond to what's on the screen
    foreach tplobj [contents] {
	append ob [$tplobj getRawData] "\n\n"
    }
    set rawData $ob
    updateLabel
}

# ------------------------------------------------------------------
#                        PRIVATE METHOD doCallBack
# ------------------------------------------------------------------
# Do the call-back stuff, plus the sound-file
itcl::body BobOB::doCallBack {cb} {
    if {[string length [string trim $cb]] != 0} {
	logLog "\nEvaluating OB call-back script '$cb'"
	set body "upvar OBS OBS; $cb"
	proc obCallBackProc {} $body
	if {[catch {obCallBackProc} msg]} {
	    logLog $msg red
	}
	logLog "OB call-back script finished"
	rename obCallBackProc {};	# should no longer be defined
    }
    if {$::bob::internal::config(bobSoundOn) && [llength $::bob::internal::config(bobObSound)] > 0} {
	::bob::bobPlaySound [bobFindSound $::bob::internal::config(bobObSound)]
    }
}
# ------------------------------------------------------------------
#                        PRIVATE METHOD editRawData
# ------------------------------------------------------------------
# The mobs-OBs get a special treatment: use the mobs-editor
itcl::body BobOB::editRawData { } {
    global ::cv ::gvar

    set mobs 1
    set count 0
    foreach child [contents] {
        if {[incr count] == 1} {
	    # If this is a MOBS-OB, then only 1st TPL needs to be shown
	    # in the classical editor (provided it is not of type GT)
	    switch -- [$child getTplType] GT {
		set showTemplates {}
	    } acquisition {
		set showTemplates $child
	    } default {
		set mobs 0
		break
	    }
	} elseif {![string equal [$child getTplId] GT]} {
	    set mobs 0
	    break
	}
    }
    if {$mobs} {
	panImportPanel bobMobs
	grab set .bobMobs
	$gvar(bobEditorTable) configure -cursor watch
	$gvar(bobEditorTnb) configure -cursor watch
    } else {
	set showTemplates [contents]
	panImportPanel bobEditor
	grab set .bobEditor
	$gvar(bobEditorTnb) configure -cursor watch
    }
    
    # first page of tabnotebook is for OBS itself
    set mfw [$gvar(bobEditorTnb) add -label $OBS(NAME)]
    pack $mfw -fill both -expand yes -padx 1 -pady 1
    
    set labels {}
    set values {}
    foreach key [array names OBS] {
	append labels "OBS.$key "
	append values \"[set OBS($key)]\" " "
    }

    BobPanedComboBox $mfw.sel -relief flat \
	    -selectbackground Grey80 -selectforeground black \
	    -background Grey90 -entryselectbackground yellow \
	    -entrybackground Grey90 -font $::bob::internal::config(bobEditorFont) \
	    -helpvariable $gvar(editorHelpVar) \
	    -leftlisthelp "Listbox with keywords of category OBS" \
	    -leftentryhelp "Keyword selected for editing" \
	    -rightlisthelp "Listbox with values for OBS keywords" \
	    -rightentryhelp "Selected keyword's value; press <Enter>\
	    to apply (<Escape> to restore, arrow keys to move up/down)"
    
    pack $mfw.sel -fill both -expand yes -pady 0 -padx 0
    # setting the -scrollbar option of the BobPanedComboBox to "auto"
    # requires the widget to be displayed (selected) when data are inserted.
    # Otherwise, the sizes returned by Tk will be wrong.
    $gvar(bobEditorTnb) select 0;	# select this page
    $mfw.sel fraction 30 70
    $mfw.sel insert [list $labels $values];	# updates needed for scrollbar
    
    # other pages are for TPLs to be shown
    foreach child $showTemplates {
	set childsite [$gvar(bobEditorTnb) add -label [$child getTplId]]
	$child init_editRawData $childsite
	$gvar(bobEditorTnb) select end;		# select this page 
	# Only works if visible, from iwidgets version 3.0 onwards
	[winfo children $childsite] fraction 30 70
    }
    
    if {$mobs} {
	$gvar(bobEditorTable) load $rawData
	$gvar(bobEditorTable) configure -helpvariable $gvar(editorHelpVar) \
		-helptext "Click on cell to edit, press <Enter> or change cell\
		to accept values" \
		-cursor {}
    }
    $gvar(bobEditorTnb) configure -cursor {}

    vwait ::bob::editingDone
    set editingDone [set ::bob::editingDone]
    switch -- $editingDone 0 - 1 { 
	# update the TPL children ("accept" or "OK" was pressed)
	set rawData [getEditedData $mfw]\n
	foreach tpl $showTemplates {
	    append rawData [$tpl getEditedData]\n
	}
	if {$mobs} {
	    append rawData [$gvar(bobEditorTable) getTableData]
	    rePopulate
	    ###puts "-------MOBS -- New OB:\n$rawData"
	} else {
	    ###puts "-------Classical Editor----New OB:\n$rawData"
	}
    }
    # Close the panel, even if "Cancel" button was pressed
    if {$mobs} {
	panClosePanel no .bobMobs
    } else {
	panClosePanel no .bobEditor
    }
}

# remark that OBS.ID has to be the first line !!!
itcl::body BobOB::getEditedData {mfw} {
    foreach {llab lent} [$mfw.sel getEntries] {}

    set maxindex [llength $llab]
    set maxindex2 [llength $lent]

    if {$maxindex != $maxindex2} {
	puts "BUG: label and entries have different size !\n\
		Please report to engineering team.\n\n\
		I try to continue - please check the value later ..."
    }
    set ob {}
    for {set i 0} {$i < $maxindex} { incr i } {
	set val [lindex $lent $i]
	set key [lindex $llab $i]
	set comp [split $key .]
	set cat [lindex $comp 0]
	set keyWoCat [join [lrange $comp 1 end] .]
	set ${cat}($keyWoCat) $val
	switch -- $key OBS.ID {
	    set ob [format $rdFormat OBS.ID $OBS(ID)]\n$ob
	} default {
	    append ob [format $rdFormat ${key} ${val}]\n
	}
    }
    updateLabel
    seq_logFitsAction OBS OBS "" CHANGE
    catch {seq_logFitsParRecord -string OBS OBS "" ID $OBS(ID)}
    return "# OB edited by BOB\n$ob\n\n"
}

# ------------------------------------------------------------------
#                        PRIVATE METHOD _sendStatus
# ------------------------------------------------------------------
itcl::body BobOB::_sendStatus { stat args } {
    if {[string equal [getObdSource] OH] || \
	    [string equal [getObdSource] templateClient]} {
    	set cmdStr "$OBS(ID) [clock format [clock seconds] \
		-format %Y-%m-%dT%H:%M:%S -gmt 1] $stat $args"
    	BobBaseObject::sendStatus ObsBlockStatus [getObdSource] $cmdStr
    }
}

# ------------------------------------------------------------------
#                        CLASS PROCEDURES
# ------------------------------------------------------------------

# Determine if there is a TPL.ID at the start of a line,
# i.e. possibly after a newline and only with whitespace in front,
# followed by whitespace and a newline or semi-colon.
# We'll check if there is in fact >1 templates, by looking for multiple
# occurences.  As we store the resulting substring-indices, we can find
# out if there is more than 1 template by looking at these index-pairs.
# Returns a list of categories' raw data
itcl::body BobOB::parseTPL { block } {
    #return a list of categories's raw data
    set ltpl {}
    set pattern "\[\t \]*TPL\\.ID\[\t \]+"
    set pattern1 "^$pattern|\n$pattern"
    set pattern2 "\n$pattern"

    set remain 1
    set tpl $block
    while {$remain} {
	if {[regexp -indices -- ($pattern1) $tpl l1]} {
	    # there is at least 1 template; get the substring range of the 1st
	    set idx1 [lindex $l1 0]
	    switch -- [string index $tpl $idx1] \n {
		# the match starts with a newline, we want to skip this newline
		incr idx1
	    }
	    # Check if there are more of these patterns following further
	    if {[regexp -indices -- ($pattern2) \
		    [string range $tpl [expr {$idx1 +1}] end] l2]} {
		# yes, there are
		set idx2 [expr {[lindex $l2 0] + $idx1}]
	    } else {
		# set the substring range of the only remaining OB
		set idx2 end
		set remain 0
	    }

	} else {
	    # no templates left
	    bobDialog "The PAF file with the set of OBs has an\n\
		    illegal format: there is no TPL.ID" application
	    break;      # break out of the "while" loop
	}
	lappend ltpl [string range $tpl $idx1 $idx2]
	if {$remain} {
	    set tpl [string range $tpl $idx2 end]
	}
    }
    return $ltpl
}

#
# ___oOo___
















