#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: bobPanedComboBox.tcl,v 1.145 2005/10/27 09:07:10 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# eallaert  1998-06-16  original version (re-write of M. Bec's work)
# eallaert  2003-11-17  replace tkListbox* by own classs procedures
# eallaert  2004-10-25  made much more general and configurable
#

#************************************************************************
#   NAME
#   BobPanedComboBox - special paned combo-box style megawidget for bob's
#   specific needs
# 
#   SYNOPSIS
#   BobPanedComboBox <widgetName> ?<option> <value>? ?<option> <value>?
#
#   DESCRIPTION
#   BobPanedComboBox creates an iTk-style mega-widget, designed especially
#   for the needs of bob. The widget consists of the following elements:
#   - a listbox (top left) to show typically the names of variables
#   - a listbox (top right) to show typically the values of variables.
#     There is be a 1-to-1 correspondence between the lines of both
#     listboxes, i.e. the name of a variable and its value are displayed
#     in the same vertical position, and any positioning action in one
#     listbox will be followed by the same action in the other.
#   - an entry field (bottom left), showing e.g. the name of the variable 
#     currently being edited
#   - an entry field (bottom right), showing the current value of the variable
#     being edited
#   - optionally a scrollbar (see -scrollbar option below); moving this 
#     scrollbar will move both listboxes
#   These elements are organized in 2 paned windows, with a vertical
#   separation bar in the middle, and a sash between the two columns,
#   allowing to redistribute the available space between the name part (left)
#   and value part (right).
#
#   PUBLIC METHODS
#   fraction <left> <right>: redistribute the space of the panes as indicated
#           by the numbers <left> and <right>. These 2 numbers must add up to
#           100.
#   getEntries : returns a list with 2 sublists, namely the list of entries
#           in the left resp. right listbox.
#   insert {<leftList> <rightList>} : append items to the end of the left and
#           right listboxes; the single argument consists of 2 sublists, one
#           for each listbox. Both sublists must have the same size
#
#   WIDGET BINDINGS
#   The normal class bindings for listboxes resp. entry fields apply, with the
#   following exceptions or modifications:
#   <Button1>: for listboxes and entry field
#       get focus, select item (for listbox and entry field)
#   <Button1-Motion>: for listboxes and scrollbar
#       drag & select
#   <Key-Prior> (= PageUp): for listboxes/entry fields
#       show previous page in listboxes
#   <Key-Next> (= PageDown): for listboxes/entry fields
#        show next page
#   <Key-Up> (= Arrow up): for listboxes/entry fields
#        select previous element in listboxes
#   <Key-Down> (= Arrow down): for listbox/entry field
#        select next element in listboxes
#   <Key-Home>: for listboxes/entry fields
#        show top of listbox resp. beginning of line
#   <Key-End>: for listboxes/entry fields
#        show end of listbox resp. end of line
#   <Key-Enter>: for entry fields
#        accept entry-field value
#   <Key-Escape>: for entry fields
#        restore entry as selected in listbox into the entry fields
#   <Key-Delete>: for listboxes/entry fields
#        delete selection
#   <Key-Insert>: for listboxes/entry fields
#        insert new element into the listboxes resp. paste copy buffer into 
#        entry field
#  
#
#   INHERITANCE
#   itk::Widget<-bobPanedComboBox
#
#   STANDARD OPTIONS
#   cursor          font           height             width
#   borderWidth     relief         selectBackground   selectForeground
#
#   See the options(n) manual entry for details on the standard options.
#
#   WIDGET-SPECIFIC OPTIONS
#   -helpvariable (name: helpVariable, class: Variable)
#       The name of the global Tcl-variable which is used to display help text
#       i.e. this widget will change the contents of this variable according
#       to the position of the cursor over its constituent widgets.
#   -leftlabel (name: leftLabel, class: Label)
#       label to display on top of the left listbox; default: empty (no label)
#   -leftlisthelp (name: leftListHelp, class: Help)
#       The short help text for the left listbox; default: empty (no help)
#   -leftentryhelp (name: leftEntryHelp, class: Help)
#       The short help text for the left entry-field; default: empty (no help)
#   -leftentrystate (name: leftEntryState, class: State)
#       Specifies the state for the left entry field. Can be "normal" (i.e.
#       editable), "disabled" or "readonly". Default: readonly
#   -listvariable (name: listVariable, class: Variable)
#       The name of the global Tcl-variable which contains the contents of
#       the 2 listboxes, if a format as returned by the getEntries method.
#       If the value of this variable changes, the widget will automatically
#       update itself to reflect the new values. Attempts to assign a
#       variable with an invalid list of sublists to -listvariable will cause
#       an error. Attempts to unset a variable in use as a -listvariable will
#       fail but will not generate an error.
#   -rightlabel (name: rightLabel, class: Label)
#       label to display on top of the right listbox; default: empty (no label)
#   -rightlisthelp (name: rightListHelp, class: Help)
#       The short help text for the right listbox; default: empty (no help)
#   -rightentryhelp (name: rightEntryHelp, class: Help)
#       The short help text for the right entry-field; default: empty (no help)
#   -rightentrystate (name: rightEntryState, class: State)
#       Specifies the state for the right entry field. Can be "normal" (i.e.
#       editable), "disabled" or "readonly". Default: normal
#   -scrollbar (name: scrollbar, class: Scrollbar)
#       Controls the presence of a vertical scrollbar to the right of the
#       entry-listbox. It can take the value "on", "off" or "auto". In the
#       latter case, the scrollbar will appear only if the listboxes contain
#       more elements than fit in a single view. Default: auto.
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#   When -scrollbar is set to "auto", the need for a scrollbar is evaluated
#   when data are inserted in the listboxed. For the listbox sizes returned 
#   by Tk to be correct, it is needed that this widget is being displayed.
#   This may not be the case if it is e.g. part of a tabnotebook page. In
#   this case, one should either make sure this page is displayed, or opt
#   for the "on" or "off" value instead of "auto".
#
#   EXAMPLES
#
#   SEE ALSO
#   options(n)
#
#------------------------------------------------------------------------
#
#
# Usual options. These are the configuration options that shall be kept
# in the composite list of configuration options if this widget is made
# part of a megawidget - to the extent that these options are not yet
# part of that composite list due to other widgets that constitute
# the megawidget.
# This set of config options is normally added to the composite list of 
# the megawidget either via the "inherit" command or via the "itk_component"
# method, as "itk_component add <widget> <creationScript>" is equivalent to
# "itk_component add <widget> <creationScript> usual".
#
usual BobPanedComboBox {
    keep -relief -width -height -borderwidth
    keep -scrollbar -helpvariable -leftlisthelp -rightlisthelp
    keep -leftlabel -rightlabel
    keep -leftentryhelp -rightentryhelp
    keep -listvariable -leftentrystate -rightentrystate
}

#
# Use option database to override default resources of base classes.
#
option add *BobPanedComboBox.height 200 widgetDefault
option add *BobPanedComboBox.width 300 widgetDefault
option add *BobPanedComboBox.borderWidth 2 widgetDefault
option add *BobPanedComboBox.relief groove widgetDefault
option add *BobPanedComboBox.entryBackground Grey90 widgetDefault
option add *BobPanedComboBox.selectForeground black widgetDefault
option add *BobPanedComboBox.selectBackground Grey80 widgetDefault
option add *BobPanedComboBox.font -Adobe-courier-medium-r-normal--14* widgetDefault

itcl::class BobPanedComboBox {
    inherit itk::Widget;	# adds 3 config-options to composite list

    constructor {args} {}
    destructor {} 

    public method fraction {args}
    public method getEntries {args}
    public method insert {args}
    public method traceListVar {args} ;	# used for the trace on listvariable
    
    private method callbackListScroll {args}
    private method callBackScrollbarScroll {args}
    private method clearEntry {args}
    private method deleteElement {args}
    private method fixScrollbar {args}
    private method getBackEntry {args}
    private method globalB1Bind {args}
    private method insertElement {args}
    private method pageUpDown {args}
    private method setHelpBind {args}
    private method showDown {args}
    private method showEnd {args}
    private method showHome {args}
    private method showUp {args}
    private method validate {args}

    private proc initSelect {args}
    private proc upDown {args}

    private variable currentListVar {};
    private variable firstEntry 1
    private variable leftFrame {}
    private variable rightFrame {}
    private variable sbVisible 0;	# visibility flag of scrollbar

    # configuration options specific to objects of this class
    itk_option define -leftentrystate leftEntryState State {readonly} {}
    itk_option define -rightentrystate rightEntryState State {normal} {}
    itk_option define -leftlabel leftLabel Label {} {}
    itk_option define -rightlabel rightLabel Label {} {}
    itk_option define -leftlisthelp leftListHelp Help {} {setHelpBind}
    itk_option define -leftentryhelp leftEntryHelp Help {} {setHelpBind}
    itk_option define -rightlisthelp rightListHelp Help {} {setHelpBind}
    itk_option define -rightentryhelp rightEntryHelp Help {} {setHelpBind}
    itk_option define -helpvariable helpVariable Variable {} {setHelpBind}
    itk_option define -listvariable listVariable Variable {} {}
    itk_option define -scrollbar scrollbar Scrollbar {auto} {}

}


# ------------------------------------------------------------------
#                        CONSTRUCTOR
# ------------------------------------------------------------------
#
itcl::body BobPanedComboBox::constructor {args} {
    itk_option add hull.width hull.height hull.borderwidth hull.relief 
    itk_component add pw {
	panedwindow $itk_interior.pw \
	    -background grey80  \
	    -borderwidth 0      \
	    -handlesize 8       \
	    -orient horizontal  \
	    -sashcursor sb_h_double_arrow \
	    -sashpad 0          \
	    -sashrelief sunken  \
	    -sashwidth 2
    } {
	# For the Iwidgets version of the panedwindow command, we
	# could in principle use the "usual" command to propagate
	# options. But that won't work if we're dealing with Tk's
	# version of panedwindow command: as the "usual" command 
	# contains a "winfo class" command to retrieve the widget's
	# class (Panedwindow), this would end up trying to apply 
	# Iwidgets' panedwindow options to Tk's. This would fail - 
	# of course, as the list of options is not identical. 
	# 
	# So we'd better give a limited set of (common) options
	# explicitly here, via "keep"
	keep -background -cursor
	keep -sashcursor -sashwidth
	keep -showhandle
    }
    ##puts "hull = $itk_component(hull) == $itk_interior"
    set leftFrame [labelframe $itk_interior.pw.leftFrame \
		       -background Grey80 -borderwidth 0 -padx 0 -pady 0 \
		       -labelanchor n]

    set rightFrame [labelframe $itk_interior.pw.rightFrame \
			-background Grey80 -borderwidth 0 -padx 0 -pady 0 \
		       -labelanchor n]

    $itk_interior.pw add $leftFrame $rightFrame
    pack $itk_interior.pw -fill both -expand yes
    $itk_component(pw) sash place 0 100 0
    # put the sash-handle close to the bottom
    ###can't do that here, as $itk_interior.pw doesn't have its final
    ###dimensions yet (and can anyway be resized)
    ###$itk_interior.pw configure -handlepad [expr {[winfo height $itk_interior.pw] -30}]
    
    #--------------------------------------------------------------------
    foreach col {left right} {
	itk_component add ${col}List {
	    listbox [set ${col}Frame].${col}List -background Grey90 \
		-selectbackground Grey80 -selectforeground black \
		-relief groove -selectborderwidth 1 -takefocus 0 \
		-exportselection 0 -height 100 \
		-selectmode single \
		-yscrollcommand [code $this callbackListScroll]
	} {
	    usual
	    ignore -height
	}
	bind $itk_component(${col}List) <Button-1> "focus $itk_component(${col}List);\
			[code $this globalB1Bind %x %y]; break"
	bind $itk_component(${col}List) <ButtonRelease-1> "[code $this globalB1Bind %x %y]; break"
	bind $itk_component(${col}List) <Button1-Motion> "[code $this globalB1Bind %x %y]; break"
	bind $itk_component(${col}List) <Key-Home> "[code $this showHome]; break"
	bind $itk_component(${col}List) <Key-End> "[code $this showEnd]; break"
	bind $itk_component(${col}List) <Key-Up> "[code $this showUp]; break"
	bind $itk_component(${col}List) <Key-Down> "[code $this showDown]; break"
	bind $itk_component(${col}List) <Key-Prior> "[code $this pageUpDown -1]; break"
	bind $itk_component(${col}List) <Key-Next> "[code $this pageUpDown 1]; break"
	# "Undocumented" binding to change the mode of entry field
	###bind $itk_component(${col}List) <Control-Shift-Key-Insert> \
	    "[code $this configure -${col}entrystate normal]; break"
	###bind $itk_component(${col}List) <Control-Shift-Key-Delete> \
	    "[code $this configure -${col}entrystate readonly]; break"
	# The bindings for Insert and Delete key are given when the
	# config-option leftentrystate is set
    }
    #-----------------------------------------------------------------
    itk_component add scrollbar {
	scrollbar $rightFrame.sbar \
	    -takefocus 0  \
	    -command [ code $this callBackScrollbarScroll ]
    } {
	usual
	ignore -takefocus
    }

    #-----------------------------------------------------------------
    foreach col {left right} {
	# First add the entry field into its corresponding frame
	itk_component add ${col}Entry {
	    entry [set ${col}Frame].${col}Entry -exportselection 0 \
		-takefocus 0 -foreground black -relief sunken \
		-highlightthickness 1 
	} {
	    usual
	    rename -background -entrybackground entryBackground Background
	    rename -selectbackground -entryselectbackground \
		entrySelectBackground Background
	    #keep -background -foreground -cursor -font
	}

	# Whether or not these entry fields are allowed to get the focus
	# depends on the itk_option -${col}entrystate, but that may not yet
	# be defined here, so we cannot decide that here. The binding
	# for <B1> is therefore set-up in the configuration of the var
	###bind $itk_component(${col}Entry) <Button-1> "focus $itk_component(${col}Entry)"
	bind $itk_component(${col}Entry) <Key-Return> [code $this validate]
	##bind $itk_component(${col}Entry) <Map> "focus $itk_component(${col}Entry)"
	bind $itk_component(${col}Entry) <Key-Up> [code $this showUp]
	bind $itk_component(${col}Entry) <Key-Down> [code $this showDown]
	bind $itk_component(${col}Entry) <Key-Prior> [code $this pageUpDown -1]
	bind $itk_component(${col}Entry) <Key-Next> [code $this pageUpDown 1]
	##bind $itk_component(${col}Entry) <Key-Delete> [code $this clearEntry]
	bind $itk_component(${col}Entry) <Key-Escape> [code $this getBackEntry]
    }

    # Order of packing is of course important !!!!
    pack $itk_component(leftEntry) $itk_component(rightEntry) -fill x  \
	-side bottom -padx 5 -pady 5
    pack $itk_component(leftList) $itk_component(rightList) -side left \
	-expand true -fill both -padx 5 -pady 5
    pack forget $itk_component(scrollbar)
    
    eval itk_initialize $args

    # don't shrink hull around components, but do the opposite
    pack propagate $itk_component(hull) 0
    if {[string equal $itk_option(-leftentrystate) "normal"]} {
	##focus $itk_component(leftEntry)
    } else {
	##focus $itk_component(rightEntry)
    }
    #!#bind $itk_component(hull) <Enter> "focus $itk_component(rightEntry)"
}
# ------------------------------------------------------------------
#                        DESTRUCTOR
# ------------------------------------------------------------------
#
itcl::body BobPanedComboBox::destructor {} {
    if {[string length $itk_option(-listvariable)] > 0} {
	uplevel \#0 trace remove variable $itk_option(-listvariable) write "{$this traceListVar}"
	uplevel \#0 trace remove variable $itk_option(-listvariable) unset "{$this traceListVar}"
    }
}

# --------------- PUBLIC CALLS --------------------------------------
itcl::body BobPanedComboBox::fraction {left right} {
    set width [winfo width $itk_component(hull)]
    $itk_component(pw) sash place 0 [expr {int($width * $left / ($left + $right))}] 0
}

itcl::body BobPanedComboBox::getEntries {} {
    set left [$itk_component(leftList) get 0 end]
    set right [$itk_component(rightList) get 0 end]
    return [list $left $right]
}

# "leftList" and "rightList" must be lists of equal length, with one
# element per scrolledEntry line.
itcl::body BobPanedComboBox::insert {list} {
    foreach {leftList rightList} $list {}
    if {[llength $leftList] != [llength $rightList]} {
	error "Bug: left and right listboxes need to have same \#elements"
    }
    eval $itk_component(leftList) insert end $leftList
    eval $itk_component(rightList) insert end $rightList
    fixScrollbar
    if {$firstEntry} {
	set firstEntry 0

	set w $itk_component(leftList)
	$w activate 0
	initSelect $w 0
	$itk_component(leftEntry) configure -state normal
	$itk_component(leftEntry) delete 0 end
	$itk_component(leftEntry) insert 0 [$w get 0]
	$itk_component(leftEntry) configure -state $itk_option(-leftentrystate)

	set w $itk_component(rightList)
	$w activate 0
	initSelect $w  0
	$itk_component(rightEntry) configure -state normal
	$itk_component(rightEntry) delete 0 end
	$itk_component(rightEntry) insert 0 [$w get 0]
	$itk_component(rightEntry) configure -state $itk_option(-rightentrystate)
    }
}

# This method is just used by the trace-callback for the listvariable
itcl::body BobPanedComboBox::traceListVar {varName index op} {
    switch -- $op write {
	# wipe old contents first
	$itk_component(leftList) delete 0 end
	$itk_component(rightList) delete 0 end
	$itk_component(leftEntry) delete 0 end
	$itk_component(rightEntry) delete 0 end
	set firstEntry 1
	if {[string length $index] == 0} {
	    insert [list [lindex [uplevel \#0 set $varName] 0] \
			[lindex [uplevel \#0 set $varName] 1]]
	} else {
	    insert [list [lindex [uplevel \#0 set [set varName]($index)] 0] \
			[lindex [uplevel \#0 set [set varName]($index)] 1]]
	}
    } unset {
	# When we get here, the variable has already been unset, and
	# there are no traces on it. First re-create the variable.
	uplevel \#0 set $itk_option(-listvariable) [list [getEntries]]
	# Ensure the subsequent configure method will execute completely
	# (i.e. as if this is the first setting of -listvariable)
	set currentListVar {};
	configure -listvariable $itk_option(-listvariable)
    }
}
			

# ------------------------------------------------------------------
#                        PUBLIC VAR leftlabel
# ------------------------------------------------------------------
itcl::configbody BobPanedComboBox::leftlabel {
    $leftFrame configure -text $itk_option(-leftlabel)
}
# ------------------------------------------------------------------
#                        PUBLIC VAR rightlabel
# ------------------------------------------------------------------
itcl::configbody BobPanedComboBox::rightlabel {
    $rightFrame configure -text $itk_option(-rightlabel)
}
# ------------------------------------------------------------------
#                        PUBLIC VAR leftentrystate
# ------------------------------------------------------------------
itcl::configbody BobPanedComboBox::leftentrystate {
    switch -- $itk_option(-leftentrystate) {
	"normal" {
	    $itk_component(leftEntry) configure -state normal
	    bind $itk_component(leftEntry) <Button-1> "focus $itk_component(leftEntry)"
	    # Only allow Insert/Delete if other listbox is also editable, and define
	    # this binding for both listboxes.
	    if {[string equal $itk_option(-rightentrystate) "normal"]} {
		foreach lb {leftList rightList} {
		    bind $itk_component($lb) <Key-Insert> "[code $this insertElement]; break"
		    bind $itk_component($lb) <Key-Delete> "[code $this deleteElement]; break"
		}
	    }
	} "disabled" - "readonly" {
	    foreach lb {leftList rightList} {
		bind $itk_component($lb) <Key-Insert> {}
		bind $itk_component($lb) <Key-Delete> {}
	    }
	    $itk_component(leftEntry) configure -state $itk_option(-leftentrystate)
	    bind $itk_component(leftEntry) <Button-1> {}
	} default {
	    error "bad leftentrystate: \"$itk_option(-leftentrystate)\" must be disabled, normal or readonly"
	}
    }
}
# ------------------------------------------------------------------
#                        PUBLIC VAR rightentrystate
# ------------------------------------------------------------------
itcl::configbody BobPanedComboBox::rightentrystate {
    switch -- $itk_option(-rightentrystate) {
	"normal" {
	    $itk_component(rightEntry) configure -state normal
	    bind $itk_component(rightEntry) <Button-1> "focus $itk_component(rightEntry)"
	    # Only allow Insert/Delete if other listbox is also editable, and define
	    # this binding for both listboxes.
	    if {[string equal $itk_option(-leftentrystate) "normal"]} {
		foreach lb {leftList rightList} {
		    bind $itk_component($lb) <Key-Insert> "[code $this insertElement]; break"
		    bind $itk_component($lb) <Key-Delete> "[code $this deleteElement]; break"
		}
	    }
	} "disabled" - "readonly" {
	    foreach lb {leftList rightList} {
		bind $itk_component($lb) <Key-Insert> {}
		bind $itk_component($lb) <Key-Delete> {}
	    }
	    $itk_component(rightEntry) configure -state $itk_option(-rightentrystate)
	    bind $itk_component(rightEntry) <Button-1> {}
	} default {
	    error "bad rightentrystate: \"$itk_option(-rightentrystate)\" must be disabled, normal or readonly"
	}
    }
}
# ------------------------------------------------------------------
#                        PUBLIC VAR listvariable
# ------------------------------------------------------------------
itcl::configbody BobPanedComboBox::listvariable {
    # Check first if we do have some work to do (vs. initialization with
    # an empty variable or configuration with same as before)
    if {[string equal $itk_option(-listvariable) $currentListVar]} {
	return
    }

    # create the var if it doesn't exist (just like Tk's listbox cmd)
    if {[string length [string trim $itk_option(-listvariable)]] > 0} {
	if {![uplevel \#0 info exists $itk_option(-listvariable)]} {
	    # it should actually be initialized with the current contents
	    # of the combobox, to behave as Tk's listbox
	    uplevel \#0 set $itk_option(-listvariable) {{{} {}}}
	}
    }
    
    # Remove the trace on the previous variable
    if {![string equal $currentListVar ""]} {
	uplevel \#0 trace remove variable $currentListVar write "{$this traceListVar}"
	uplevel \#0 trace remove variable $currentListVar unset "{$this traceListVar}"
    }

    # set a trace for writing into this variable
    uplevel \#0 trace add variable $itk_option(-listvariable) write "{$this traceListVar}"
    # set a trace for unsetting this variable
    uplevel \#0 trace add variable $itk_option(-listvariable) unset "{$this traceListVar}"

    # now trigger the trace
    traceListVar $itk_option(-listvariable) {} write
}
# ------------------------------------------------------------------
#                        PUBLIC VAR scrollbar
# ------------------------------------------------------------------
itcl::configbody BobPanedComboBox::scrollbar {

    switch -- $itk_option(-scrollbar) {
	on - off - auto {
	    fixScrollbar
	}
	default {
	    error "bad value for -scrollbar: \"$itk_option(-scrollbar)\"\
			should be \"on\", \"off\" or \"auto\""
	}
    }
}
# --------------- PRIVATE STUFF --------------------------------------



itcl::body BobPanedComboBox::callbackListScroll {args} {
    eval $itk_component(scrollbar) set $args
    eval $itk_component(rightList) yview moveto [lindex $args 0]
    eval $itk_component(leftList) yview moveto [lindex $args 0]
}

itcl::body BobPanedComboBox::callBackScrollbarScroll {args} {
    eval $itk_component(leftList) yview $args
    eval $itk_component(rightList) yview $args
}

itcl::body BobPanedComboBox::clearEntry {} {
    $itk_component(rightEntry) delete 0 end
}

itcl::body BobPanedComboBox::deleteElement {} {
    foreach column {left right} {
	set w $itk_component(${column}List)
	set index [$w index active]
	$w delete $index 
	fixScrollbar
	$w activate $index;	# activate new entry
	$w see active
	initSelect $w active
	$itk_component(${column}Entry) delete 0 end
	$itk_component(${column}Entry) insert 0 [$w get $index]
    }
}

itcl::body BobPanedComboBox::fixScrollbar {args} {
    switch $itk_option(-scrollbar) {
	on   {set sbState 1}
	off  {set sbState 0}
	auto {
	    update idletasks
	    # we can only rely on [$w yview] if the widget is being displayed
	    if {[string equal [$itk_component(rightList) yview] {0 1}]} {
		set sbState 0
	    } else {
		set sbState 1
	    }
	}
    }
    if {$sbState != $sbVisible} {
	if {$sbState} {
	    pack $itk_component(scrollbar) -fill y -expand yes \
		    -padx 5 -pady 5 -after $itk_component(rightList)
	} else {
	    pack forget $itk_component(scrollbar)
	}
	set sbVisible $sbState
    }
    if {[string length $args] != 0} {
	eval $itk_component(scrollbar) configure $args
    }
}

itcl::body BobPanedComboBox::getBackEntry {} {
    foreach side {left right} {
	if {[string equal $itk_option(-${side}entrystate) "normal"]} {
	    $itk_component(${side}Entry) delete 0 end
	    set origContent [$itk_component(${side}List) get active]
	    $itk_component(${side}Entry) insert 0 $origContent
	    ##$itk_component(${side}Entry) selection range 0 end
	}
    }
}

itcl::body BobPanedComboBox::globalB1Bind {x y} {

    foreach col {left right} {
	set w $itk_component(${col}List)
	set index [$w nearest $y]
	$w activate $index
	initSelect $w $index
	$itk_component(${col}Entry) configure -state normal
	$itk_component(${col}Entry) delete 0 end
	$itk_component(${col}Entry) insert 0 [$w get $index]
	$itk_component(${col}Entry) configure -state $itk_option(-${col}entrystate)
    }
}

itcl::body BobPanedComboBox::insertElement {} {
    foreach column {left right} {
	set w $itk_component(${column}List)
	set index [$w index active]
	$w insert $index "<New entry>"
	fixScrollbar
	$w activate $index;	# activate new entry
	$w see active
	initSelect $w active
	$itk_component(${column}Entry) delete 0 end
	$itk_component(${column}Entry) insert 0 [$w get $index]
	$itk_component(${column}Entry) selection range 0 end
    }
    if {[string length $itk_option(-listvariable)] > 0} {
	uplevel \#0 set $itk_option(-listvariable) \
	    [list [list [$itk_component(leftList) get 0 end] \
		       [$itk_component(rightList) get 0 end]]]
    }
}

itcl::body BobPanedComboBox::pageUpDown {amount} {
    $itk_component(rightList) yview scroll $amount pages
    $itk_component(leftList) yview scroll $amount pages
}

itcl::body BobPanedComboBox::setHelpBind {} {
    if {[string length $itk_option(-helpvariable)] == 0} {
	# if running within "panel", the following should work
	set widgetRoot [string range $itk_component(rightList) \
			    0 [string first . $itk_component(rightList) 1]]
	set indices [array names ::shortHelp_line]
	if {[set idx [lsearch -glob $indices *${widgetRoot}*]] >= 0} {
	    # Next will cause another pass through this method, but that's OK
	    configure -helpvariable ::shortHelp_line([lindex $indices $idx])
	}
    } 
    if {[string length $itk_option(-leftlisthelp)] != 0} {
	bind $itk_component(leftList) <Enter> \
	    "set $itk_option(-helpvariable) [list $itk_option(-leftlisthelp)]"
	bind $itk_component(leftList) <Leave> \
	    "set $itk_option(-helpvariable) {}"
    }
    if {[string length $itk_option(-rightlisthelp)] != 0} {
	bind $itk_component(rightList) <Enter> \
	    "set $itk_option(-helpvariable) [list $itk_option(-rightlisthelp)]"
	bind $itk_component(rightList) <Leave> \
	    "set $itk_option(-helpvariable) {}"
    }
    if {[string length $itk_option(-leftentryhelp)] != 0} {
	bind $itk_component(leftEntry) <Enter> \
	    "set $itk_option(-helpvariable) [list $itk_option(-leftentryhelp)]"
	bind $itk_component(leftEntry) <Leave> \
	    "set $itk_option(-helpvariable) {}"
    }
    if {[string length $itk_option(-rightentryhelp)] != 0} {
	bind $itk_component(rightEntry) <Enter> \
	    "set $itk_option(-helpvariable) [list $itk_option(-rightentryhelp)]"
	bind $itk_component(rightEntry) <Leave> \
	    "set $itk_option(-helpvariable) {}"
    }
}

itcl::body BobPanedComboBox::showDown { } {
    foreach col {left right} {
	upDown $itk_component(${col}List) 1
	$itk_component(${col}Entry) configure -state normal
	$itk_component(${col}Entry) delete 0 end
	$itk_component(${col}Entry) insert 0 [$itk_component(${col}List) get active]
	$itk_component(${col}Entry) configure -state $itk_option(-${col}entrystate)
    }
}

itcl::body BobPanedComboBox::showEnd {} {
    $itk_component(rightList) yview moveto 1
    $itk_component(leftList) yview moveto 1
}

itcl::body BobPanedComboBox::showHome {} {
    $itk_component(rightList) yview moveto 0
    $itk_component(leftList) yview moveto 0
}

itcl::body BobPanedComboBox::showUp {} {
    upDown $itk_component(rightList) -1
    upDown $itk_component(leftList) -1

    $itk_component(leftEntry) configure -state normal
    $itk_component(leftEntry) delete 0 end
    $itk_component(leftEntry) insert 0 [$itk_component(leftList) get active]
    $itk_component(leftEntry) configure -state $itk_option(-leftentrystate)

    $itk_component(rightEntry) configure -state normal
    $itk_component(rightEntry) delete 0 end
    $itk_component(rightEntry) insert 0 [$itk_component(rightList) get active]
    $itk_component(rightEntry) configure -state $itk_option(-rightentrystate)
    ###$itk_component(rightEntry) selection range 0 end
}

itcl::body BobPanedComboBox::validate {} {
    set activeindex [$itk_component(rightList) index active]

    # Remark that we accept *all* input, incl. blanks!

    # The entry widgets takes care that editing can only be done
    # when in state "normal", so we don't need to check that here.
    if {[string equal $itk_option(-leftentrystate) normal]} {
	$itk_component(leftList) insert active [$itk_component(leftEntry) get]
	$itk_component(leftList) delete active
	$itk_component(leftList) activate $activeindex
	initSelect $itk_component(leftList) $activeindex
    }
    if {[string equal $itk_option(-rightentrystate) normal]} {
	$itk_component(rightList) insert active [$itk_component(rightEntry) get]
	$itk_component(rightList) delete active
	$itk_component(rightList) activate $activeindex
	initSelect $itk_component(rightList) $activeindex
    }
    if {[string length $itk_option(-listvariable)] > 0} {
	uplevel \#0 set $itk_option(-listvariable) \
	    [list [list [$itk_component(leftList) get 0 end] \
		       [$itk_component(rightList) get 0 end]]]
    }

}

#-------------------------------------------------------------
#                    CLASS PROCEDURES
#------------------------------------------------------------
# This procedure begins the process of making a selection in the listbox. 
#
# Arguments:
# w -  The listbox widget.
# el - The element for the selection operation (in numerical form).
# (Based on "tk::ListboxBeginSelect", Tcl/Tk 8.4.7)
itcl::body BobPanedComboBox::initSelect {w el} {
    $w selection clear 0 end
    $w selection set $el
    $w selection anchor $el
    event generate $w <<ListboxSelect>>
}

# Moves the location cursor (active element) up or down by one element,
# and changes the selection.
#
# Arguments:
# w -      The listbox widget.
# amount - +1 to move down one item, -1 to move back one item.
# (Based on "tk::ListboxUpDown", Tcl/Tk 8.4.7)
itcl::body BobPanedComboBox::upDown {w amount} {
    $w activate [expr {[$w index active] + $amount}]
    $w see active
    $w selection clear 0 end
    $w selection set active
    event generate $w <<ListboxSelect>>
}
#
# ___oOo___




