#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: bobStoreInArray.tcl,v 1.145 2005/10/27 09:07:08 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# eallaert  29/04/1997  created
# eallaert  07/11/1997  simplified
#

#************************************************************************
#   NAME
#   bobStoreInArray - creates data array from raw OBD data
# 
#   SYNOPSIS
#   bobStoreInArray <string> <cat> ?<maxIdx>? ?<ext>?
# 
#   DESCRIPTION
#   bobStoreInArray will search in the Observation Block Description 
#   <string> for data that match a certain category <cat>, and, if found, 
#   store these data in an array called <cat> (possibly followed by an index:
#   legal categories consist of 3 alphabetic characters, followed optionally
#   by a numeric index). Categories which permit indices can have such indices
#   ranging from 1 to <maxIdx> (default: 0 - no indices)
#
#   The argument <ext> here is used to be able to store also in a second
#   array, typically used for the values of the template signature file.  If
#   set, the storage happens in the array <cat><ext> (possibly with the index
#   in between).
#
#   This procedure cannot be part of the bobBobBaseObject class, as it needs
#   to be called by more specific classes (e.g. TPL), with their arrays being
#   private.
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#   The list of categories in <string> matching <cat> are returned.  This is
#   an empty list if there are no such categories in <string>.  It can be
#   more than 1 value if the particular <cat> appears with indices in <string>.
#
#   If <string> has a syntax error, an error will be returned with the
#   corresponding error message.
#
#   CAUTIONS
#   This proc requires that an array with name equal to <cat> (possibly 
#   followed by an index) exists in the context of the caller!
#   It also requires that the array "names" exist at one stack level higher!
#
#   EXAMPLES
#
#   SEE ALSO
#   ICD between the VLT Control SW and the Observation Handling Subsystem
#   (VLT-ICD-ESO-17240-19200)
#
#   BUGS     
#
#------------------------------------------------------------------------
#
# Trick to store element in array in one go by evaluating a whole line.
# data is stored in OB or TPL with this uplevel.
# Watch out with some of the names of the variables which are upvar-ed: they
# can only be changed if that's done simultaneously in several places!!
# I.e. this is the case for "names" (this latter var is used to keep a list
# of keys in the exact order as encountered in the OBD; this is needed
# as [array names CAT] returns a list in an order chronologically unrelated
# to the creation of the array elements)
proc bobStoreInArray {string cat {maxIdx 0} {ext {}}} {
    # 1st step: get rid of continuation marks (i.e. concatenate)
    set pattern "\\\\\n"
    regsub -all -- $pattern $string " " string

    # 2nd step: replace "CATx." by "CAT x "
    set pattern "\[\t \]*${cat}"
    regsub -all -- (^$pattern)(\[0-9\]*)\.|(\n$pattern)(\[0-9\]*)\. \
	    $string {\1 \2 \3 \4 } string

    # 3rd step: delete all other lines (including comment & blank lines)
    set pattern1 "\[\t \]*\[A-Z\]\[A-Z\]\[A-Z\]\[0-9\]*\\.\[^\n\]*"
    set pattern2 "\[\t \]*#\[^\n\]*"
    set pattern "($pattern1|$pattern2)"
    regsub -all -nocase -- ^${pattern}|\n$pattern $string {} string
    set pattern "\n(\[\t \]*\n)+"
    regsub -all -- $pattern $string "\n" new
    set string [string trim $new]

    # 4th step (needed for Tcl 8.0 list-handling): replace "; by " ;
    regsub -all -- \"\; $string "\" ;" string

    set idx [lindex $string 1];	# if numeric, then we have indices
    switch -- {} $idx return;	# means resulting string is empty
    set withIdx [string is integer -strict $idx]

    # get all indices for this cat, so we know what arrays are involved
    foreach el [split $string \n] {
	switch -- {} [set idx [lindex $el 1]] continue
	if {![string is integer -strict $idx]} {
	    if {$withIdx} {
		error "found keywords of category $cat with and without indices"
	    }
	} else {
	    lappend indices $idx
	    set withIdx 1
	}
    }
    if {![info exists indices]} {
	set indices {{}}
	set new1 $string
    } else {
	set indices [lrmdups $indices]
	if {[set m [eval max $indices]] > $maxIdx} {
	    if {$maxIdx} {
		error "found keyword of category ${cat}$m\n\
			($cat can only have indices from 1 to $maxIdx)"
	    } else {
		error "found keyword of category ${cat}$m\n\
			(no indices permitted for $cat)"
	    }
	}
    }
    foreach idx $indices {
	switch -- $idx {} {} default {
	    # step 4: for index x, convert "CAT x " to "CATx "
	    set p1 "\[\t \]*${cat}"
	    set p2 "\[ \]+"
	    set p3 "${idx} "
	    regsub -all -- (^$p1)($p2)($p3)|(\n$p1)($p2)($p3) $string \
		    {\1\3\4\6 } new
	    # step 5: remove all other lines
	    set pattern "\[\t \]*${cat} \[^\n\]*"
	    regsub -all -nocase -- ^${pattern}|\n$pattern $new {} new1
	    set new1 [string trimleft $new1]; # 1st line removal leaves eol
	}
	regsub -all -- \"\; $new1 "\" \;" new
	set CATx ${cat}${idx}
	upvar names names
	upvar ${CATx}${ext} array
	if {![uplevel 1 info exists ${CATx}${ext}]} {
	    ##error "Bug: ${CATx}${ext} array variable does not exist"
	}
	foreach line [split $new \n] {
	    lassign $line category key value
	    set array($key) $value
	    lappend names(${CATx}) $key
	}   
	lappend cats $CATx
    }
    return $cats
}

# The last part is actually a newer version of what looked before as follows;
 
# escape chars that may cause troubles for script evaluation
#	regsub -all -- {\$|\[|\]|\{|\}} $new1 {\\&} new
#	set CATx ${cat}${idx}
#	proc $CATx {key val} "upvar 2 ${CATx}${ext} array; \
#		upvar 2 names names ; \
#		set array(\$key) \$val; \
#		lappend names(${CATx}) \$key "
#	if {[catch {eval $new} msg]} {error $msg}
#	lappend cats $CATx

# This worked fine, but is about 20% slower than the current implementation
# (which requires not to have blank lines!).