#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: feditReadDefFile.tcl,v 2.11 2003/11/12 09:10:35 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# rschmutz  19/04/99  lrange replaced with regexp (for TclTk 8.0).
# mcomin    09/11/95  created
# mcomin    04/12/95  improved check on starting char in line1
#           27/10/97  remove substitution of specific chars
#

#************************************************************************
#   NAME
#   
#   feditReadDefFile - Reads a file containing error or log format definitions
#
#   SYNOPSIS
#   feditReadDefFile  <what> <file>  <mode>
#
#   DESCRIPTION
#
#   The procedure opens an error/log  definition file and stores the 
#   information in lists.  
#
#   The error definition is split over three lines with the following format :
#
#   -  errorNumber errSeverity
#   -  error Format
#   -  errHelpFile
#
#   The log definition is split over two lines with the following format :
#
#   -  logNumber logType
#   -  logFormat
#
#   The information stored in the definition file is stored line-by-line 
#   into a vector, whose elements have the following structure :
#
#          index |  Comment
#         ......................................
#
#            0      Byte offset
#            1      free
#
#                   LOG          ERROR 
#
#            2      logId        errNumber
#            3      logType      errSeverity
#            4      logMnemonic  errMnemonic
#            5      logFormat    errFormat
#            6                   errHelpFile
#            7                   errOffset
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#

proc feditReadDefFile { what file  mode } {
   global par offsetRec errMemo recList totalElem 
   
   #
   #  Initialization
   #
   
   catch {unset recList offsetRec }

   set offsetRec(none) {none}
   set par(EDIT_MODE)  $mode
   set recList(0)      {}   
   set totalElem       0
   set warnFlag        0
   
   # Common Indexes
   
   global rec_offset rec_id  rec_type rec_memo rec_format
            
   # Indexes for error description structure
   
   global err_help  err_offset
  
   #  Open / Read the file and fill the lists
   
   set fh [open $file $mode]; 
        
   while {[set nc [gets $fh tmp]] != -1} {
                 
      set line1 [string trim $tmp " "]
      set bytes [tell $fh] ;               # Get current byte offset         
      set c     [string index $line1 0] ;  # Get first character of the line
           
      if { $c == "" || $c == "#" || $c == "\n" || $nc == 0 } {
          continue  ;# Skip comments etc ..
      } 
          
      #
      #  Read first line of the error/log definition
      #  - Error Number and Severity
      #  - log Id and Type
      #
       
      incr totalElem ;                          # Update error counter
      set  myId        [lindex $line1 0]   
      set  myType      [lindex $line1 1] 
      set par(LAST_ID) $myId  

      if { $what == "ERRORS" } {
         #
         # Process error offset definition in case of error definition file   
         #     
         if { $myId == "define"} { 
            set name  [lindex $line1 1]
            set value [lindex $line1 2]
            set offsetRec($name) $value ;      #  Save error offset definition
            continue
         } 
              
         set myType [string index $myType 0 ] ; # Patch for LCC compatibility
      
         if { [set len [llength $line1]] > 2 } {      
                set errOffset  [lindex $line1 2]
         } else {
                set errOffset  {}      
         }          
      }
            
      #
      #  Read second line of the error/log definition
      #  - Error Mnemonic and Format
      #  - Log Mnemonic and Format
      #
      
      if {[gets $fh line2] < 0} {
         set tmp    "Cannot Read File Definition : "
         append tmp "File must end with Carriage Return"

         if {[info exists par(SH_AREA)]} {
            par(SH_AREA) displayText $tmp
	 } else {
            puts $tmp
         }
         return FAILURE
      }

      # set  tmp      [lrange $line2 0 0];              # Extract Mnemonic
      regexp {^[^ ]+} $line2 tmp		      # Extract Mnemonic
      set  myMemo   [string trim $tmp " :"];          # Cut leading " :"      
      set  first    [string first ":" $line2];        # Locate the first ':'
      incr first     
      set  tmp      [string range $line2 $first end]; # Extract string from ':'
      set  myFormat [string trim  $tmp ];

      if {[set idx [string first "%s" $myFormat ]] != -1 } {
         incr warnFlag +1
      }

      # Initialize array record 
      
      set recList($myId) [list $bytes "" $myId $myType $myMemo $myFormat ]
      
      if {$what == "ERRORS"} {
         #
         #  Get Error Help File
         #
         if {[gets $fh errHelp ] < 0} { 

            if {[info exists par(SH_AREA)]} {
               $par(SH_AREA) displayText "Cannot Read Error Help File"
	    } else {
               puts "Cannot Read Error Help File"
            }

            return 
         }
         
         lappend recList($myId)  $errOffset $errHelp ; # Set Data Structures
         set errMemo($myMemo)    $myId        
      }
      
      # DEBUG information
#      puts "DBG > recList($myId) = $recList($myId)" ; puts "\n"
   }

   close $fh;                     # Close the file 

   if {$warnFlag > 0 } {
      puts "\nWARNING : Found string format specifier \"%s\" in <$file>"
      puts "          It is safer to SPECIFY FIELD LENGTH -e.g.- \"%.80s\""
   }

   catch {unset recList(0)} ;# Take away dummy element
}

proc feditCheckAll {} {
   global par recList rec_format 

   set counter 0

   foreach item [lsort -integer [array names recList]] {
      set buf $recList($item)

      if {[set tmp [feditCheckLog [lindex $buf $rec_format] Quotes]] != ""} {
         set recList($item) [lreplace $buf $rec_format $rec_format $tmp]
         incr counter +1
      }
   }

   feditDisplay ERRORS

   if {$counter == 0} {
      $par(SH_AREA) displayText "NO Substitution DONE"
   } else {
      incr par(SAVE_FLAG) +1
      $par(SH_AREA) displayText "Numb. $counter Format Substitution DONE"
   }
}

#------------------------------------------------------------------------
# PROCEDURE : feditCheckLog - The procedure checks the correctness of the
# the log syntax and replaces some special tcl/tk characters with others.
#------------------------------------------------------------------------

proc feditCheckLog { text pippo } {

   set flag 0

   # String definition
   incr flag [regsub -all {%s} $text {%.80s} tmp1]

   # Substitute ; with :
#   incr flag [regsub -all {\;} $tmp1 {:} tmp2]

   # Substitute "" with ''
#   incr flag [regsub -all {\"} $tmp2 {'} tmp1]

   # String definition
#   incr flag [regsub -all {\[} $tmp1 "\(" tmp2]

   # String definition
#   incr flag [regsub -all {\]} $tmp2 "\)" tmp1]

   # Take away '$' signs
#   incr flag [regsub -all { \$} $tmp1 " \\$" tmp2]

   if {$flag != 0 } {
       return $tmp1
   } else {
       return ""
   }
}


#
# ___oOo___
