#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: osrtdToffset.tcl,v 1.20 2004/01/23 15:21:49 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# pbiereic  15/02/00  created
# pbiereic  08/06/00  added clearToffset
# pbiereic  11/07/00  added option -deviceName
# pbiereic  05/02/02  bug fixed in method check_coords 
#
# osrtdToffset.tcl -- Defines parameters for a telescope offset

#************************************************************************
#   NAME
#   osrtdToffset.tcl - itk class to define a telescope offset
# 
#   SYNOPSIS
#   osrtdToffset <path> ?options?
# 
#   DESCRIPTION
#   osrtdToffset allows to define interactively the image coordinates for
#   a telescope offset
#
#   FILES
#
#   ENVIRONMENT
#   CCS
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#   RTD documentation
#
#   BUGS     
#
#------------------------------------------------------------------------

itk::usual osrtdToffset {}

class ::osrtdToffset {
    inherit util::TopLevelWidget

    #  constructor: create a new instance of this class

    constructor {args} {
	eval itk_initialize $args
	wm protocol $w_ WM_DELETE_WINDOW "$this cancel"
	# make the control panel
	make_control
	make_help
	wm resizable $w_ 0 0
	set lineId_ -1
    }

    # create the Toffset frame window

    method make_control {} {
        pack [frame $w_.status -borderwidth 2 -relief groove] \
            -side top -fill none -expand 0

	blt::table $w_.status
	foreach el {startx starty endx endy} {
	    itk_component add $el {
		LabelEntry $w_.status.$el \
		    -text "[string toupper $el]: " \
		    -command [code $this setValues] \
		    -validate numeric \
                    -anchor e \
		    -relief sunken \
		    -valuewidth 8 \
		    -labelwidth 8
	    }
	}

	blt::table $w_.status \
		$itk_component(startx)     1,0 -padx 10 \
		$itk_component(starty)     2,0 \
		$itk_component(endx)       1,1 \
		$itk_component(endy)       2,1

        pack [frame $w_.buttons -borderwidth 2 -relief groove] \
            -side top -fill x -expand 1

	blt::table $w_.buttons
	foreach el {ok clear toSlit cancel} {
	    itk_component add $el {
		button $w_.buttons.$el \
		    -text "[string toupper $el]" \
		    -pady 3 \
		    -width 8 \
		    -command [code $this $el]	    
	    }
	}
	global ::$w_.center
	itk_component add center {
	    checkbutton $w_.buttons.center \
		-text "To center of [cget -deviceName]" \
		-width 0 \
		-variable $w_.center \
		-onvalue 1 -offvalue 0 \
		-anchor w \
		-pady 3 \
		-bd 2 -relief raised \
		-command [code "$this slit_center"]
        }
	config -slitcenter [cget -slitcenter]
	slit_center

	blt::table $w_.buttons \
		$itk_component(cancel)     2,3 \
		$itk_component(toSlit)     2,2 \
		$itk_component(clear)      2,1 \
		$itk_component(ok)         2,0 \
		$itk_component(center)     1,0 -columnspan 4
    }

    # short help text

    method make_help {} {
	add_short_help $itk_component(cancel) \
	    {CANCEL: {bitmap b1} = Cancel setting telescope offset}
	add_short_help $itk_component(ok) \
	    {OK: {bitmap b1} = Set telescope offset}
	add_short_help $itk_component(clear) \
	    {CLEAR: {bitmap b1} = Clear all arrows used for setting telescope offset}
	add_short_help $itk_component(center) \
	    {To center of slit: {bitmap b1} = Offset to center of slit}
	add_short_help $itk_component(toSlit) \
	    {TOSLIT: {bitmap b1} = Offset to center of slit}
	foreach el {startx starty endx endy} {
	    add_short_help $itk_component($el) \
		"[string toupper $el]: {bitmap b1} = Image/detector coordinate"
	}
    }

    # Clear all arrows used for setting telescope offset

    method clear {} {
	$draw_ deselect_objects
	make_line
    }

    # send offset parameters to DCS

    method ok {} {
	set id $lineId_
	lassign [update_status] x0 y0 x1 y1
	set_arrow_shape $arrow2 $id
	rtdbReply "$x0 $y0 $x1 $y1"
	cancel 0
    }

    method toSlit {} {
	set_center
	update_status
    }

    # lower window and delete line

    method cancel {{with_ack 1}} {
	if {$with_ack} {
	    rtdbReply -1
	}
	wm withdraw $w_
	deselect_line
	$draw_ set_drawing_mode anyselect	
    }

    method slit_center {} {
	global ::$w_.center
	set slit_center_ [set $w_.center]
	$draw_ set_check_done $slit_center_
    }

    # method which is called when values are entered

    method setValues {args} {
	foreach el {startx starty endx endy} {
	    set $el [$itk_component($el) get]
	    if {"[set $el]" == "" || [set $el] <=0 } {
		error_dialog "Illegal value in \"[string toupper $el]\""
		return
	    }
	}
	lassign [check_coords $startx $starty $endx $endy] x0 y0 x1 y1
	if {$lineId_ == -1} {
	    lassign [rtdbCanvasToChip $x0 $y0 $x1 $y1 $target_image_] x0 y0 x1 y1
	    $draw_ create_object $x0 $y0
	    $draw_ update_line $x1 $y1
	    $draw_ check_done $x1 $y1
	} else {
	    lassign [rtdbChipToCanvas $x0 $y0 $x1 $y1 $target_image_] x0 x1 y0 y1
	    $target_canvas_ coords $lineId_ $x0 $x1 $y0 $y1
	    $draw_ adjust_line_selection $lineId_
	    update_status
	}
    }

    # Draw a line via the draw widget

    method make_line {} {
	rtdbPopup $w_
	clearAll
	# wm deiconify $draw_
	$draw_ set_arrowtype last
	$draw_ set_drawing_mode line "$w_ notify_cmd"
	if {$slit_center_} {
	    $draw_ set_check_done 1
	}
    }

    method clearAll {} {
	deselect_line
	$target_canvas_ delete toffset
	$itk_component(ok) config -state disabled
	$itk_component(toSlit) config -state disabled
	foreach el {startx endx starty endy} {
	    $itk_component($el) config -value ""
	}
    }

    # deselect line

    method deselect_line {} {
	if {$lineId_ == -1} {
	    return
	}
	catch {after cancel $afterId_}
        $draw_ remove_notify_cmd $lineId_
	set_arrow_shape $arrow1 $lineId_
	set lineId_ -1
	$draw_ deselect_objects
    }


    # This method is called when the line was drawn, moved, resized or deleted. 

    method notify_cmd {op args} {
	#puts "Toffset: $op $args"

	if {"$op" == "move" || "$op" == "resize"} {
	    update_status
	} elseif {"$op" == "delete"} {
	    if {$lineId_ != -1} {
		cancel
	    }
	} elseif {"$op" == "scale"} {
	    update_status
	} elseif {"$op" == "new_image"} {
	    if {[winfo viewable $w_]} {
		clear
	    } else {
		clearAll
	    }
	} else {
	    set lineId_ $op
	    $target_canvas_ addtag toffset withtag $lineId_
	    $itk_component(ok) config -state normal
	    lassign $args x0 y0 x1 y1
	    if {"[cget -slitcenter]" != ""} {
		$itk_component(toSlit) config -state normal
	    }
	    if {$slit_center_ != 0 && "[cget -slitcenter]" != ""} {
		set_center
	    }
	    $draw_ add_notify_cmd $lineId_ "$w_ notify_cmd" 1
	    update_status
	    #blink_arrow
	}
    }
    
    method set_center {} {
	lassign [cget -slitcenter] x1 y1
	lassign [$target_canvas_ coords $lineId_] cx0 cy0 cx1 cy1
	lassign [rtdbChipToCanvas $x1 $y1 0 0] cx1 cy1 d d
	$target_canvas_ coords $lineId_ $cx0 $cy0 $cx1 $cy1
	$draw_ adjust_line_selection $lineId_
    }

    # check coordinates. Enter with x0,y0,x1,y1. Returns updated x0,y0,x1,y1.

    method check_coords {x0 y0 x1 y1} {
        set iw [$target_image_ width]
        set ih [$target_image_ height]
        lassign [rtdbImageToChip $iw $ih 0 0] iw ih d d
	set x0 [rtdbCheckRange $x0 1 $iw]
	set x1 [rtdbCheckRange $x1 1 $iw]
	set y0 [rtdbCheckRange $y0 1 $ih]
	set y1 [rtdbCheckRange $y1 1 $ih]
	return "$x0 $y0 $x1 $y1"
    }

    # update status display and return window parameters

    method update_status {} {
	if {$lineId_ == -1} {
	    return
	}
        lassign [line_image_coord] x0 y0 x1 y1
	# check that frame is within max. size and in main image
	lassign [check_coords $x0 $y0 $x1 $y1] x0 y0 x1 y1
	lassign [format "%6.2f %6.2f %6.2f %6.2f" $x0 $y0 $x1 $y1] x0 y0 x1 y1

	$itk_component(startx) config -value $x0
	$itk_component(endx)   config -value $x1
	$itk_component(starty) config -value $y0
	$itk_component(endy)   config -value $y1

	return "$x0 $y0 $x1 $y1"
    }

    # blink arrowshape

    method blink_arrow {} {
	if {$lineId_ == -1} {
	    return
	}
	incr arrowshape
	if {$arrowshape > 1} {
	    set arrowshape 0
	    set shape $arrow1
	} else {
	    set shape $arrow2
	}
	if {! [set_arrow_shape $shape $lineId_]} {
	    set afterId_ [after 1000 "$w_ blink_arrow"]
	}
    }

    # set arrow shape

    method set_arrow_shape {shape id} {
	return [catch {$target_canvas_ itemconfigure $id -arrowshape $shape}]
    }

    # image coordinates of the arrow

    method line_image_coord {} {
	lassign [$target_canvas_ coords $lineId_] x0 y0 x1 y1
	# convert to pixel coordinates
	lassign [rtdbCanvasToChip $x0 $y0 $x1 $y1 $target_image_] x0 y0 x1 y1
	return "$x0 $y0 $x1 $y1"
    }

    # method to clear the arrow of the previous toffset

    method clearToffset {} {
	catch {$target_canvas_ delete toffset}
    }
    
    # -- public vars --
    
    # target rtdimage
    itk_option define -target_image target_image Target_image {} {
	# get internal widget names for target image
	set target_image_ [$itk_option(-target_image) get_image]
	set target_canvas_ [$itk_option(-target_image) get_canvas]
	set draw_ [$itk_option(-target_image) component draw]
    }

    # set the state to normal/disabled to enable/disable widgets
    itk_option define -state state State {disabled} {}

    # set the width for  displaying labels and values
    itk_option define -labelwidth labelWidth LabelWidth 5
    itk_option define -valuewidth valueWidth ValueWidth 10

    # pixel coordinate of slit center
    itk_option define -slitcenter slitcenter Slitcenter {""} {
	if {[info exists itk_component(center)]} {
	    if {"[cget -slitcenter]" != ""} {
		$itk_component(center) config -state normal
	    } else {
		$itk_component(center) config -state disabled
	    }
	}
    }

    # name of device
    itk_option define -deviceName deviceName DeviceName {slit} {
	if {[info exists itk_component(center)]} {
	    set name [cget -deviceName]
	    $itk_component(center) config -text "To center of $name"
	    add_short_help $itk_component(center) \
		    "To center of $name: {bitmap b1} = Offset to center of $name"
	}
    }

    # -- protected vars --

    # canvas id of the line object used to position the tel. offset
    protected variable lineId_
    
    # target internal rtdimage
    protected variable target_image_ 

    # canvas widget for main image and region object marking frame.
    protected variable target_canvas_ 

    # CanvasDraw object, for setting up move, resize operations on embedded image
    protected variable draw_

    # internal rtdimage for Toffset frame
    protected variable image_

    # canvas window containing Toffset frame image, 
    # different than target_canvas for popup frames
    protected variable canvas_

    # offset to center of slit, flag
    protected variable slit_center_ 0

    # id of after script
    protected variable afterId_

    # current arrowshape
    protected variable arrowshape 0
    protected variable arrow1 {8 10 3}
    protected variable arrow2 {8 12 9}
}

# ___oOo___
