#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: rtdbPrivUtil.tcl,v 1.24 2005/06/19 19:36:29 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# pbiereic  15/02/00  created
# pbiereic  30/05/01  added proc rtdbGetCfgPath to locate the PAF file
#                     (VLTSW20010227)
# pbiereic  05/11/03  fixed: raiseRtdWin during startup

#************************************************************************
#   NAME
#   rtdbPrivUtil.tcl - internal utility procs for the base RTD application
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#   The following proc's are contained in this file
#   - rtdbSetEnv:         set environment if not already defined
#   - rtdbHandleCfgFile:  Handle config file and set global variables
#   - rtdbGetMenu:        read user defined menu file
#   - rtdbReadPaf:        Read PAF file and set the keys into the global array rtdb
#   - rtdbCheckRange      checks range and return value which is within limits
#   - rtdbRoundCoords     round coordinates
#   - rtdbCenterBox       return coordinates of a box in the center
#   - raiseRtdWin         Raise RTD window
#   - defwin              Define window
#   - pick                Pick object
#   - waitNextImage       wait until next image is displayed
#   - dumpImage           dump current image to file
#   - loadFile            load a file
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#   RTD documentation
#
#   BUGS     
#
#------------------------------------------------------------------------

# force loading of conversion routines
rtdbConvGen

# set environment unless already defined

proc rtdbSetEnv {environ name} {
    global ::rtdb ::env

    if {! [info exists env($environ)] && [info exists rtdb($name)]} {
	rtdbFeedback "Setting environment $environ to: $rtdb($name)"
	set env($environ) $rtdb($name)
    }
}

# get pathname of a PAF file located either in $INS_ROOT or $VLTROOT
# (in that order). Returns full pathname if found, else the filename.

proc rtdbGetCfgPath {cfgFile} {
    global ::rtdb ::env

    if { [file isfile $cfgFile] } {
	return $cfgFile
    }

    set dirname "$env(VLTROOT)/config"
    if { [info exists env(INS_ROOT)] } {
	set cfgdir "$env(INS_ROOT)/$env(INS_USER)/COMMON/CONFIGFILES"
	if { [file isfile "$cfgdir/$cfgFile"] } {
	    set dirname $cfgdir
	}
    }
    return "$dirname/$cfgFile"
}

# Handle config file and set global variables

proc rtdbHandleCfgFile {} {
    global ::rtdb ::env

    set rtdbCfg RTDB.cfg
    if {[info exists env(CFGFILE)]} {
	set rtdbCfg $env(CFGFILE)
	unset env(CFGFILE)
    }
    set rtdb(STRTOPT) " "

    rtdbReadPaf $rtdbCfg

    # process the keys defined in the config file
    rtdbCheckGenKeys

    catch {
	set text [read_file "$env(VLTDATA)/tmp/$rtdb(APPNAME)\-$env(USER)"]
	lassign $text pid host port
	set rtdbFd [server_connect -nobuf $host $port]
	puts "Application $rtdb(APPNAME) exists...sending raiseRtdWin command"
	catch {
	    etcl $rtdbFd "raiseRtdWin"
	    close $rtdbFd
	}
	rtdbTerminate
    }
    rtdbFeedback "Name of the application is set to: $rtdb(APPNAME)"

    rtdbSetCameraName
    rtdbReadMenuConfigFiles
    rtdbReadCtrlConfigFiles
    rtdbCcsInit
    rtdbGetPlugins
    rtdbGetMisc
}

# general keywords which can be specified in config file or
# overriden by command line options

proc rtdbCheckGenKeys {} {
    global ::rtdb ::env

    set APPNAME rtdb
    set APPVERS "??"
    set APPDESC "??"
    foreach el "APPNAME APPVERS APPDESC" {
	if {[info exists env($el)]} {
	    set rtdb($el) $env($el)
	} elseif {! [info exists rtdb($el)]} {
 	    set rtdb($el) [set $el]
	}
    }
}

# camera name

proc rtdbSetCameraName {} {
    global ::rtdb ::env
    rtdbSetEnv RTD_CAMERA CAMERA
    if {[info exists env(RTD_CAMERA)]} {
	append rtdb(STRTOPT) " -camera $env(RTD_CAMERA)"
	# start rtdServer
	if { ! [catch {exec sh -ic {rtdServer 2> /dev/null &} >& /dev/null &} pid]} {
	    catch {wait -nohang $pid}
	}
    }
}

# read menu config files

proc rtdbReadMenuConfigFiles {} {
    global ::rtdb
    set i 1
    while {[info exists rtdb(MENUFIL$i)]} {
	if {! [info exists rtdb(MENUKEY$i)]} {
	    rtdbExit_dialog "Found RTDB.MENUFIL$i but not RTDB.MENUKEY$i in RTDB.cfg"
	}
	rtdbGetMenu $rtdb(MENUFIL$i) $rtdb(MENUKEY$i)
	unset rtdb(MENUFIL$i) rtdb(MENUKEY$i)
	incr i
    }
}

# read widget-control config file

proc rtdbReadCtrlConfigFiles {} {
    global ::rtdb

    set i 1
    while {[info exists rtdb(CTRLFIL$i)]} {
	if {! [info exists rtdb(CTRLKEY$i)]} {
	    rtdbExit_dialog "Found RTDB.CTRLFIL$i but not RTDB.CTRLKEY$i in RTDB.cfg"
	}
	rtdbGetCtrls $rtdb(CTRLFIL$i) $rtdb(CTRLKEY$i)
	unset rtdb(CTRLFIL$i) rtdb(CTRLKEY$i)
	incr i
    }
}

# ccs Init

proc rtdbCcsInit {} {
    global ::rtdb

    set rtdb(CCSINIT) [rtdbGetLVal CCSINIT]
    set rtdb(CCS_FILE) ""
    if {$rtdb(CCSINIT)} {
	rtdbFeedback "Initializing CCS. Application name: $rtdb(APPNAME)"
	if {[catch {seq_ccsInit $rtdb(APPNAME)} msg]} {
	    warning_dialog $msg
	    set rtdb(CCSINIT) 0
	} else {
	    set rtdb(CCS_FILE) $msg
	}
    }
}

# remember which plugins to load in rtdbRtd.tcl

proc rtdbGetPlugins {} {
    global ::rtdb ::auto_path ::env

    set anames [array names rtdb]
    set n [llength $anames]
    for {set i 0} {$i < $n} {incr i} {
	if {[set idxfile [lsearch -exact $anames PLGFILE$i]] == -1} { continue }
	set file "$rtdb([lindex $anames $idxfile])"
	if {! [file isfile $file] || [info exists env(RTDB_TEST_ROOT)]} {
	    # check if file can be found on $VLTROOT
	    set f1 [file split $file]
	    set nf [llength $f1]
	    set fx [lrange $f1 [expr {$nf - 2}] [expr {$nf - 1}]]
	    set f1 [eval file join "/" $env(VLTROOT) "lib" $fx ]
	    if {! [file isfile $f1] } {
		rtdbExit_dialog "Plugin file $file not found"
	    }
	    set file $f1
	}
	lappend auto_path [file dirname $file]
	rtdbFeedback "Plugin to load: $file"
	append rtdb([winfo name .]PLUGIN) "$file\:"
	unset rtdb([lindex $anames $idxfile])
    }
}

# process other miscelleneous keys in config file

proc rtdbGetMisc {} {
    global ::rtdb ::env
    # attach to camera variable
    set rtdb(ATTACH) [rtdbGetLVal ATTACH]

    # for backwards compatibility with irdcs, isrtd and nrtd
    if {! [info exists env(IRTD_PORT)]} {
	foreach var "RTDIR_PORT ISRTD_PORT NRTD_PORT NIRTD_PORT" {
	    if {[info exists env($var)]} {
		set env(IRTD_PORT) $env($var)
		rtdbFeedback "environment env(IRTD_PORT) was set to env($var) = $env($var)"
	    }
	}
    }
}

# read user defined menu file and add menu list to rtdb(ADD_MENUS)

proc rtdbGetMenu {file ctgry} {
    global ::rtdb

    rtdbReadPaf $file $ctgry
    set keys [lsort [array names rtdb]]
    foreach key $keys {
	if {! [regsub {MENU[0-9]*\.*} $key {} var]} { continue }
	if {[scan $key "MENU%d\." idx] != 1} { continue }
	set ${var}($idx) "[set rtdb($key)]"
	unset rtdb($key)
    }
    foreach idx [lsort -integer [array names NAME]] {
	set cmd "menu [set NAME($idx)]"
	foreach name "TYPE LABEL CMD SHELP" {
	    if {! [info exists ${name}($idx)]} {
		append cmd " {}"
	    } else {
		append cmd " {[set ${name}($idx)]}"
	    }
	}
	lappend rtdb(ADD_MENUS) "$cmd"
	# puts $cmd
    }    
}

# read user defined control file and add definitions to rtdb(ADD_CTRLS)

proc rtdbGetCtrls {file ctgry} {
    global ::rtdb

    rtdbReadPaf $file $ctgry
    set keys [lsort [array names rtdb]]
    foreach key $keys {
	if {! [regsub {CTRL[0-9]*\.*} $key {} var]} { continue }
	if {[scan $key "CTRL%d\." idx] != 1} { continue }
	if {[scan $var "OPT%d" idx2] != 1} {
	    set ${var}($idx) "[set rtdb($key)]"
	} else {
	    append opt($idx) "[set rtdb($key)] "
	}
	unset rtdb($key)
    }
    foreach idx [lsort -integer [array names NAME]] {
	set cmd [set NAME($idx)]
	foreach name "TYPE SHELP INDX" {
	    if {! [info exists ${name}($idx)]} {
		append cmd " {}"
	    } else {
		append cmd " [set ${name}($idx)]"
	    }
	}
	if {[info exists opt($idx)]} {
	    lappend cmd $opt($idx)
	} else {
	    lappend cmd ""
	}
	lappend rtdb(ADD_CTRLS) "$cmd $ctgry"
	# puts $cmd
    }
}

# Read PAF file and set the keys with the category RTDB into the global
# array rtdb e.g. the value of the keyword RTDB.FILE1 will be stored in rtdb(FILE1)

proc rtdbReadPaf {file {ctgry RTDB}} {
    global ::env ::rtdb

    set file [rtdbGetCfgPath $file]

    if {! [file isfile $file] } {
	rtdbExit_dialog "Could not find file $file"
    }

    rtdbFeedback "Processing config file $file ... (category $ctgry)"
    #rtdbFeedback "Processing config file [lindex [file split $file] end]... (category $ctgry)"
    set lctgry [string length $ctgry]
    set nctgry [expr {$lctgry + 1}]

    # prepare for env. variable substitution
    foreach el [array names env] {
	set $el $env($el)
    }

    # read PAF file and set the keywords
    set buffer [read_file $file]
    foreach line [split $buffer "\n"] {
	catch {eval set $line}
    }

    # strip off the category and set the key in the global array rtdb
    foreach var [lsort [info vars]] {
	set n [string length $var]
	if {$n > $nctgry && "[string range $var 0 $lctgry]" == "$ctgry\."} {
	    set rtdb([string range $var $nctgry [expr {$n - 1}]]) [set $var]
	}
    }
}

# return logical value of a key (0 for false, 1 for true)

proc rtdbGetLVal {name} {
    global ::rtdb

    if {! [info exists rtdb($name)]} { return 0 }
    set val $rtdb($name)
    if {"[string toupper [string index $val 0]]" == "T"} { return 1 }
    return 0
}

# return the index list of key RTDB.$ctrl

proc rtdbIdx {ellist ctrl} {
    global ::rtdb

    # generate default index list
    set defaultList [rtdbSetList [llength $ellist]]
    if {! [info exists rtdb($ctrl)]} { return $defaultList }  ; # all indices
    set indices $rtdb($ctrl)
    if {[regexp -- " -1" $indices]} { return $defaultList }
    regsub -all "," $indices " " indices
    # puts "rtdbIdx: $indices"
    return [string trim $indices " "]
}

# return a generated list of numbers "1 2 ... $range"

proc rtdbSetList {range} {
    for {set i 1} {$i <= $range} {incr i} {
	append list "$i "
    }
    return $list
}

# checks range and return value which is within limits

proc rtdbCheckRange {val min max} {
    if {$val < $min} {
        return "$min"
    } elseif {$val > $max} {
        return "$max"
    } else {
        return "$val"
    }
}

# round coordinates

proc rtdbRoundCoords {args} {
    foreach el $args {
	lappend list [expr {round($el)}]
    }
    return $list
}

# return coordinates of a box in the center

proc rtdbCenterBox {w h size} {
    set x0 [expr {round((double ($w) - $size) /2.)}]
    set y0 [expr {round((double ($h) - $size) /2.)}]
    set x1 [expr {round(double ($x0) + $size)}]
    set y1 [expr {round(double ($y0) + $size)}]
    return "$x0 $y0 $x1 $y1"
}

proc rtdbMenuCall {args} {
    global ::rtdb
    eval $rtdb(Rtd) $args
}

# interface procedures callable via SCRIPT command.

# Raise RTD window

proc raiseRtdWin {} {
    global ::rtdb
    rtdbPopup $rtdb(Rtd)
    return 0
}

# Define window

proc defwin {args} {
    global ::rtdb
    rtdbCall $rtdb(Rtd) defwin
}

# Pick object

proc pick {args} {
    global ::rtdb
    eval rtdbCall $rtdb(Rtd) pick [join $args]
}

# wait until next image is displayed

proc waitNextImage {} {
    global ::rtdb
    rtdbCall $rtdb(Rtd) wait_next_image
}

# dump current image to file

proc dumpImage {file} {
    global ::rtdb
    rtdbCall $rtdb(Rtd) save_file $file
}

# load a FITS file

proc loadFile {file} {
    global ::rtdb ::env
    if {[string first "/" $file] == -1 && [info exists env(INS_ROOT)]} {
	set file "$env(INS_ROOT)/SYSTEM/DETDATA/$file"
    }
    set img [$rtdb(RtdImage) get_image]
    if {[catch {$img config -file $file}]} {
	return -1
    }
    return 0
}

# return compatible command for itcl_info

proc rtdbItclInfo {} {
    global ::tcl_version
    if { $tcl_version >= 8.3 } {
	return "::itcl::find"
    } else {
	return "itcl_info"
    }
}

# ___oOo___
