#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: uifOutputIcon.tcl,v 3.49 2005/09/07 11:46:01 vltsccm Exp $" 
#
# uifOutput.tcl
#
# who         when      what
# --------   --------   ----------------------------------------------
# M.COMIN    22/03/95   created 
# M.COMIN    31/07/95   Use args instead of config
# M.COMIN    15/09/95   Trim " " from strings
# mcomin     05/10/95   Add public variables for the event mechanism
#
# mcomin     06/03/96   Porting to Tk4.0 and itcl2.0
#            13/05/96   Update man page
#            21/04/98   Add options -label, -labelWidth, -labelFont
#
#************************************************************************
#   NAME
#     UifOutputIcon - Class for VLT output icon according to 
#                     the values of a variable
#   SYNOPSIS
#     UifOutputIcon <widget-name>
#   DESCRIPTION
#
#     UifOutputIcon defines the class for the VLT output widget.
#     The widget is configured to associate an Icon to a range of values.
#     It compares the variable current value with those ranges to detect
#     in which range the current value falls into and display the proper 
#     icon
#
#   COMPONENTS
#     The itk 'component' method returns the component's full name.
#
#     uifOutputIcon - Label widget
#
#   METHODS
#
#   PUBLIC VARIABLES
#
#   WIDGET OPTIONS
#
#     Only the options related to the VLT conventions are listed here.
#     The complete set can be obtained throught the config function.
#
#     Inherits options from UifOutputWidget class and supports :
#
#     anchor  left ,center  Determine where how the value should be displayed
#             or right      within the widget.
#     frame    <1/0>        Switch on/off the frame around the widget.
#     label      <string>   String used to label the widget
#     labelFont  <string>   New font for the label
#     labelWidth <integer>  Label width : can be used to align several
#                           widgets with different labels in columns
#     listPictures <list>   List of icons associated to the specific range.
#     listValues  <list>    List of threshold values. The current value
#                           of the <variable> is compared with these values
#                           to determine the range.
#     variable <string>     Name of the variable associated to the widget
#
#   CAUTIONS
#
#   EXAMPLES
#
#     Create an instance
#
#     UifOutputString  .w -variable gvar(myVar) \
#                         -listValues {10 15 50 } \
#                         -listPictures {icon1.gif icon2.gif icon3.gif} 
#
#     In this example the result would be :
#
#                         |  Icon
#       ==============================
#             value <= 10 |  icon1     
#       10 <  value <= 15 |  icon2       
#       50 <  value       |  icon3      
#
#    Re-configure the widget
#        .w configure -frame 0
#
#    Other operations
#        set myVar    [.w cget -variable]   Get a widget's resource
#
#   SEE ALSO
#     uifWidget  UifOutputWidget UifOutput UifOutputLed UifOutputArray
#   BUGS
#
#------------------------------------------------------------------------
#/*

itk::usual UifOutputIcon { }

class UifOutputIcon {
    inherit UifOutputWidget
    
    constructor {args} {}
    
    itk_option define -variable  variable  Variable  ""  
    itk_option define -frame     frame     Frame     0
    itk_option define -label     label     Label     ""
    itk_option define -listPictures listPictures ListPictures {}
    itk_option define -listValues  listValues  ListValues  {}
    
    public method setValue     { refValue }
    public method getImageInfo {}
  
    # Extract first element of the lists
    
    private variable minValue    0
    private variable maxValue    1

    # Extract last element of the lists
    
    private variable emptyImage  ""
    private variable listImages  {}
    private variable imgFile     ""
    private variable imgIDX      ""

    protected variable wthis     ""

}; #end of class definition

#------------------------------------------------------
# Constructor
#------------------------------------------------------

body UifOutputIcon::constructor { args } { 
    
    set wthis  [namespace tail $this]

    itk_component add uifOutputIcon {
	label $itk_interior.uifOutputIcon -bd 2 -anchor nw
    } {
	keep -relief -anchor -background
    }
    
    itk_component add label {
	label $itk_interior.label -anchor w -relief flat
    } {
	rename -width -labelWidth labelWidth LabelWidth
	rename -font  -labelFont labelFont LabelFont
    }
    
    pack $itk_component(uifOutputIcon)  -side left -anchor w -fill x
    itk_initialize
    
    # Set default configuration 
    
    set emptyImage [image create photo emptyImage -width 25 -height 25]
    
    $itk_component(uifOutputIcon) configure -image  $emptyImage
    eval configure -frame 1 $args
}

#---------------------------------------------------------------
# METHOD : getImageInfo
#---------------------------------------------------------------

body UifOutputIcon::getImageInfo {} {
    set img   [$itk_component(uifOutputIcon) cget -image]
    return    [list $imgFile $imgIDX $img]
}

#---------------------------------------------------------------
# METHOD : setValue
#---------------------------------------------------------------

body UifOutputIcon::setValue { refValue } {

    if { $refValue <= $minValue} {
	set imgIDX 0
	set myImage [lindex $listImages 0]
    } elseif { $refValue >= $maxValue} {
	set imgIDX end
	set myImage [lindex $listImages end]
    } else {
	
	set max [llength $itk_option(-listValues)]
	for { set i 0 } { $i < $max } { incr i } {
	    set val1 [lindex $itk_option(-listValues) $i];
	    set val2 [lindex $itk_option(-listValues) [expr {$i + 1} ]];
	    
	    if { $refValue >= $val1 && $refValue < $val2 } {
		set imgIDX  $i
		set myImage [lindex $listImages $i]
		break
	    }
	}
    }

    set imgFile [lindex $itk_option(-listPictures) $imgIDX]

#    puts "UifOutputIcon  setValue $refValue  > $myImage"

    $itk_component(uifOutputIcon) configure -image $myImage

    update
}

#---------------------------------------------------------------
# OPTION : -label
#---------------------------------------------------------------

configbody UifOutputIcon::label {
    if {[cequal $itk_option(-label) ""] } {
	pack forget $itk_component(label)
    } else {
	$itk_component(label) configure -text $itk_option(-label)
	pack $itk_component(label) -side left -before $itk_component(uifOutputIcon)
    }
}

#---------------------------------------------------------------
# OPTION : -variable
#---------------------------------------------------------------

configbody  UifOutputIcon::variable {
    addToDataList [list $itk_option(-variable)]
    setTraceVar   [list $itk_option(-variable)]
    
    if { ! [cequal $itk_option(-variable) ""]} {
	setValue [uplevel #0 set [list $itk_option(-variable)]]
    }
}

     
#---------------------------------------------------------------
# OPTION : -frame
#---------------------------------------------------------------

configbody  UifOutputIcon::frame {
    if { $itk_option(-frame) } {
	configure -relief ridge
    } else {
	configure -relief flat
    }
}

#---------------------------------------------------------------
# OPTION : -listPictures
#--------------------------------------------------------------- 

configbody UifOutputIcon::listPictures {

    set idx 0
    foreach img $itk_option(-listPictures) {

	if {[cequal $img ""]} {
	    set itmp $emptyImage
	} else {
	    set imgFile   [findVLTFile bitmaps/$img]
	    set imgFormat [string trimleft [file extension $img] "."]

	    # The name of the images are prefixed with the widget name otherwise
	    # there are clashes when several widget are in the same panels

	    set itmp [image create photo ${wthis}_img${idx} -file $imgFile -format $imgFormat]
	}

	lappend listImages $itmp
	incr idx
    }
}

#---------------------------------------------------------------
# OPTION : -listValues
#---------------------------------------------------------------

configbody UifOutputIcon::listValues {

    if { [llength $itk_option(-listValues)] != 0} {
	set minValue [lindex $itk_option(-listValues) 0]
	set maxValue [lindex $itk_option(-listValues) end]
    }
}

