#************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: classCommon.tcl,v 3.49 2005/09/07 11:45:53 vltsccm Exp $"
#
# who       when        what
# --------  ----------  ----------------------------------------------
# mcomin    30/01/96    created
# mcomin    09/03/96    Ported to Tk4.0 and itcl 2.0
#           13/05/96    Update man page
#           04/07/96    Change config. procedure for -nameInstance
# eallaert  1999-02-15  Ported to Tcl/Tk 8.0 and itcl 3.0
#
#************************************************************************
#   NAME
#     classCommon - Parent class for classContainer and classFrame
#   SYNOPSIS
#     inherit classCommon
#   DESCRIPTION
#     The classCommon is a virtual class containing all methods and public
#     variables common to classContainer and classFrame
#   COMPONENTS
#
#   METHODS
#     getClassPath    
#        Returns the full class path.
#   PUBLIC VARIABLES
#
#   WIDGET OPTIONS
#
#     background   <color>    Color of the border, if the class is visible,
#                             otherwise color of the background.
#     classSelect  <index>    Index of the class displayed by the widget.
#                             This ranges from 0 to (N - 1), where N is the
#                             total number of classes.
#     length       <val>      Widget length
#
#     nameInstance <string>   Name of the instance.
#
#     show         <0/1>      Visibility flag  
#
#     wdgConfig    <list>     For each class there is a list of configuration 
#                             parameters which are invoked when the class is 
#                             displayed.
#                             If the class does not need configuration 
#                             parameters, the corresponding list must be empty.
#
#     wdgLibrary   <list>     List of library names where the classes to be
#                             be displayed are stored.
#                             At run time the search path follows the VLT
#                             searching rules : module, INTROOT, VLTROOT.
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS
#
#------------------------------------------------------------------------
#

class classCommon {
   inherit itk::Widget 

   constructor {args} {
       global ::tcl_version
       eval itk_initialize $args
       set wthis [namespace tail $this]
   }

   itk_option define  -canvasAnchor canvasAnchor CanvasAnchor nw
   itk_option define -helparea     helparea     Helparea     ""
   itk_option define -show         show         Show          0
   itk_option define -wdgLibrary   wdgLibrary   WdgLibrary    ""
   itk_option define -wdgConfig    wdgConfig    WdgConfig     ""
   itk_option define -classSelect  classSelect  ClassSelect   0
   itk_option define -nameInstance nameInstance Instance      "myInstance"
   itk_option define -wdgClassList wdgClassList WdgClassList  ""

   public method getClassPath {} {
      return $myInstance
   }

   public method display {} {
      if {[winfo exists $myInstance]} { pack $myInstance -padx 2 -pady 2 }
   } 

   public method hide    {} {
      if {[winfo exists $myInstance]} { pack forget $myInstance}
   }

   public method deleteInstance  {}
   public method wdgClassDisplay {wdgClass uifFlag}

   protected method checkSelection {}

   protected variable initialized  0
   protected variable frameName    ""
   protected variable nChoices     0
   protected variable wdgConfigLen 0
   protected variable myInstance   ""
   protected variable oldSelect    -1
   protected variable oldInstance  ""
   protected variable wthis        ""
   protected variable isMapped     0
   protected variable uifFlag      0
}; # end of class definition



#--------------------------------------------------------------------
# METHOD : checkSelection
#--------------------------------------------------------------------
body classCommon::checkSelection {} {

    if { $nChoices == 0 } { return -1 }

    if { $itk_option(-classSelect) >= $nChoices } {
      set tmp "Wrong Value ($itk_option(-classSelect)) for option -classSelect"
      append tmp "\nValid range is 0 , [incr nChoices -1]"
      tk_dialog .dialog "Dialog Box" $tmp  info 0 OK

      return -1 
   }

   return $itk_option(-classSelect)
}


#--------------------------------------------------------------------
# METHOD : deleteInstance
#--------------------------------------------------------------------
body classCommon::deleteInstance {} {
   foreach w [winfo children $frameName] {
      delete object  $w ;# Destroy previus frames ... if any
      update ;# Flush X events
   }
}

#--------------------------------------------------------------------
# METHOD : wdgClassDisplay
#--------------------------------------------------------------------

body classCommon::wdgClassDisplay {wdgClass uifFlag } {

#   puts "DBG> wdgClassDisplay ... myInstance = ${frameName}.$itk_option(-nameInstance)"

   # If the class to be displayed is already selected , just return.
   set myInstance  ${frameName}.$itk_option(-nameInstance)    

   if {$itk_option(-classSelect) == $oldSelect } {
       if {[winfo exists $myInstance]} {
	   set isMapped 1; return 
       }
   }

   # Create an instance of the class

   set isMapped 0
   set oldSelect   $itk_option(-classSelect)
   deleteInstance

   #######################################
   # Create a new instance
   #######################################

   # Create a new instance and  Force look & Feel
   
   if { $itk_option(-classSelect) < $wdgConfigLen } {
       set tmpConfig [lindex $itk_option(-wdgConfig) $itk_option(-classSelect)] 
   } else {
       set tmpConfig [lindex $itk_option(-wdgConfig) end]
   }

#   puts  "classCommon::wdgClassDisplay  tmpConfig   $tmpConfig"


   if {[catch {eval $wdgClass $myInstance -relief flat -helparea \"$itk_option(-helparea)\" $tmpConfig } ret]} {
       tk_dialog .dialog "Dialog Box" "Error Loading $wdgClass\n$ret" info 0 OK
       return
   }

   if {$uifFlag} { eval $myInstance display }
}


#--------------------------------------------------------------------
#  OPTION : -nameInstance
#--------------------------------------------------------------------

configbody classCommon::nameInstance {
   if {![cequal $itk_option(-nameInstance) $oldInstance]} { deleteInstance }
   set oldInstance $itk_option(-nameInstance)
}

#--------------------------------------------------------------------
#  OPTION : -show
#--------------------------------------------------------------------

configbody classCommon::show {
   uplevel #0 set $wthis.ckShow $itk_option(-show)
   if {$initialized && [winfo exists $myInstance]} { $this show }
#    if {$initialized } { $this show }
}

#--------------------------------------------------------------------
#  OPTION : -wdgLibrary
#--------------------------------------------------------------------

configbody classCommon::wdgLibrary {
   global ::auto_path
   foreach item $itk_option(-wdgLibrary) {
      if {[lsearch $auto_path "*/${item}.tcl"] == -1 } {
         set libPath [findVLTFile lib/${item}.tcl]
	 if {![cequal $libPath "" ]} { lappend auto_path $libPath }
      }
   }
}

#--------------------------------------------------------------------
#  OPTION : -wdgConfig
#--------------------------------------------------------------------

configbody classCommon::wdgConfig {
   set wdgConfigLen [llength $itk_option(-wdgConfig)]

#   puts "classCommon::wdgConfig  len $wdgConfigLen  '$itk_option(-wdgConfig)'"
}

#--------------------------------------------------------------------
#  OPTION : -classSelect
#--------------------------------------------------------------------

configbody classCommon::classSelect {

#   puts "DBG> classCommon::classSelect select = $itk_option(-classSelect)"
   if {[cequal $itk_option(-classSelect) "" ] } { return }
   uplevel #0 set $wthis.rbChoice $itk_option(-classSelect)
   if { $initialized } { $this show }
}

#--------------------------------------------------------------------
#  OPTION : -wdgClassList
#--------------------------------------------------------------------

configbody classCommon::wdgClassList {

   set nChoices [llength $itk_option(-wdgClassList)]
   if { $initialized } {
      deleteInstance
      $this show 
   }
}

#___oOo___



