#************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: classContainer.tcl,v 3.49 2005/09/07 11:45:58 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# santin    08/01/96  created : basic code from uvesSingleContainer
# mcomin    10/01/96  Review of the man-page
#                     Re-design of some parts
#                     Merged with singlContainer functionality
# mcomin    12/01/96  Take away 'default' switch for fonts
# mcomin    15/01/96  Add check if class is uifClass or not
#                     wdgLibrary supports list of libs.
# mcomin    22/01/96  Fixed bug in wdgLibrary pub. var.
# mcomin    09/02/96  Improved version after testing with CCD
# mcomin    02/04/96  Ported to Tk4.0 and itcl 2.0
#           08/05/96  Add missing options : relief, bd
#           13/05/96  Update man page
#           20/05/96  Add -borderwidth widget option
#           21/05/96  Add method setShow
#           21/08/96  Optimization 
#
#************************************************************************
#   NAME
#     classContainer - Container for classes
#   SYNOPSIS
#     classContainer <widget-name>
#   DESCRIPTION
#
#     The classContainer is a widget that can be configured to interactively
#     display a class out of a set of classes.
#
#     The widget is formed by :
#
#     - A checkButton : used to enable/disable the display of the class.
#     - A Title : the configuration changes according to whether it should
#       display a single class or a set of classes.
#
#       In the latter case the title is formed by a list of radioButtons 
#       allowing the user to select the class to be displayed.
#
#     - A frame containing the class to be displayed.
#
#     Length and height are used when no class is visible in order to keep
#     its dimensions , otherwise the container assumes the size of the 
#     included class.
#   COMPONENTS
#
#   METHODS
#     The class inherits the methods of classCommon and provides :
#
#     show 
#        Force the classContainer to display the class or not according
#        to the current value of the 'show' visibility flag.
#        (The 'show' visibility flag in inherited from the classCommon)
#
#     enableRadioButton <radio button list index>
#        Enables the radioButtons specified in the list.
#        The first radio button index start with 0
#
#     disableRadioButton <radio button list index>
#        Disables the radioButtons specified in the list.
#        The first radio button index start with 0
#
#     getRbVariable
#        Returns the name of the variable associated to the radioButtons.
#
#     enableRadioButton <radio button list index>
#        Enables the radioButtons specified in the list.
#        The first radio button index start with 0
#
#   PUBLIC VARIABLES
#
#   WIDGET OPTIONS
#
#     The class inherits the widget options of classCommon and supports :
#
#     height       <integer>    Widget height
#
#     titleFont    <font>       Font for the labels of the radioButtons.
#                               The following alias are valid:
#                               - normal, big, small
#                               - bold, bigbold, smallbold
#
#     titleList    <name list>	List of labels for the radioButtons
#                               If this list  contains a single element
#                               the widget is configured as 'single-class'
#                               container otherwise it is configure as 
#                               'multi-class' container.
#
#     wdgClassList <name list>  List of classes to be displayed.
#
#   CAUTIONS
#
#   EXAMPLES
#
#     Create an instance
#
#     classContainer .w -background red -titleFont bold \
#                       -wdgLibrary libtestClass \
#                       -titleList {"Class#1" "Class#2 "Class#3"} \
#                       -wdgClassList {xx_uifClass yy_uifClass zz_uifClass}\
#                       -wdgConfig    { {-cwp a:b} {-cwp a:c} {-cwp a:d} }
#    
#     Re-configure the widget
#        .w configure -classSelect 2      Display class #2
#        .w configure -show 0             Set visibility OFF
#        .w configure -background blue    Change color
#
#     Other operations
#        .w getClassPath                  Get full class path name
#        .w disableRadioButtons 0 2       Disable first and third radioButton
#        set lib [.w cget -wdgLibrary]    Get a widget's resource
#
#   SEE ALSO
#     classCommon  classFrame
#   BUGS
#
#------------------------------------------------------------------------
#

itk::usual classContainer { }

class classContainer {
   inherit classCommon 

   constructor {args} {}

   itk_option define -titleFont titleFont TitleFont default {
       if {$initialized} { setFont }
   }

   itk_option define -titleList titleList TitleList ""
   itk_option define -showCommand showCommand ShowCommand ""

   public  method show    {}
   public  method setShow {}
   public  method setFont {}
   public  method getRbVariable {}
   public  method enableRadioButton  { args }
   public  method disableRadioButton { args } 
   public  method classReConfigure   { configList }
  
}; # end of class definition

#------------------------------------------------------
# Constructor
#------------------------------------------------------

body classContainer::constructor {args} {

   itk_component add f1 {
      frame $itk_interior.f1
   } 

   itk_component add container {
      frame $itk_interior.container
   } {      
       rename -background -classBackground classBackground ClassBackground
       keep -width -height -relief -borderwidth
   }

   itk_component add ck {
       UifCheckButton $itk_component(f1).ck -variable ${wthis}.ckShow \
	   -command "$wthis setShow" -default 1 -onVal 1 -offVal 0 
   } {
       keep -background
   }
   
   itk_component add title {
       label $itk_component(f1).title
   } {
       keep -font -foreground 
   }

   itk_component add rbBox {
      frame $itk_component(f1).rbBox
   }

   pack $itk_component(f1) $itk_component(container) -fill both
   pack $itk_component(ck) $itk_component(title)  $itk_component(rbBox) -side left 

   uplevel #0 set $wthis.ckShow 0
   set frameName $itk_component(container)

   # This option must be processed first to know specific class configurations

   if {[set idx [lsearch $args "-wdgConfig"]] != -1} {
       configure -wdgConfig [lindex $args [expr {$idx +1}]]
       set args [lreplace $args $idx [expr {$idx +1}]]
   }

   eval itk_initialize -height 20 -width 50 -borderwidth 2 -relief groove $args

   $this show
   $this setFont
   set initialized 1
}

#---------------------------------------------------------------
# METHOD : getRbVariable
#---------------------------------------------------------------

body classContainer::getRbVariable {} {
   return [$itk_component(rbBox).rb0 cget -variable]
}

#---------------------------------------------------------------
# METHOD : enableRadioButton
#---------------------------------------------------------------

body classContainer::enableRadioButton { args } {
   foreach item $args {
       if {$item < $nChoices } {
	  $itk_component(rbBox).rb${item} configure -state normal
       }
   }
}

#---------------------------------------------------------------
# METHOD : disableRadioButton
#---------------------------------------------------------------

body classContainer::disableRadioButton { args } {
   foreach item $args {
       if {$item < $nChoices } {
	  $itk_component(rbBox).rb${item} configure -state disabled
       }
   } 
}

#---------------------------------------------------------------
# METHOD : setShow
#---------------------------------------------------------------

body classContainer::setShow {} {
   global ::${wthis}.ckShow
#   puts "classContainer::setShow   configure -show  (${wthis}.ckShow) [set ${wthis}.ckShow]"
   configure -show [set ${wthis}.ckShow]
}


#---------------------------------------------------------------
# METHOD : setFont
#---------------------------------------------------------------

body classContainer::setFont {} {

   if {[cequal $itk_option(-titleFont) ""]} { return }

   switch $itk_option(-titleFont) {
      big        {set newFont -Adobe-helvetica-medium-r-normal--24*}
      small      {set newFont -Adobe-helvetica-medium-r-normal--12*}
      bold       {set newFont -Adobe-helvetica-bold-r-normal--14*}
      bigbold    {set newFont -Adobe-helvetica-bold-r-normal--24*}
      smallbold  {set newFont -Adobe-helvetica-bold-r-normal--12*}
      default    {set newFont -Adobe-helvetica-medium-r-normal--14*}
   }

   if {$nChoices > 1 } {
      foreach w [winfo children $itk_component(rbBox)] {
         $w configure -font $newFont
      }
   } else {
      $itk_component(title) configure -font $newFont ;      # singleContainer
   }
}

#--------------------------------------------------------------------
# METHOD : classReConfigure
#--------------------------------------------------------------------
   
body classContainer::classReConfigure { configList } {
       
   set myInstance  ${frameName}.$itk_option(-nameInstance)    
       
   #   puts "DBG> classReConfigure ... $myInstance  [llength $configList] $configList"
       
   # Re-configure single class configuration parameters used to
   # switch between classes.
   $this configure -wdgConfig $configList
       
   if { $itk_option(-classSelect) < $wdgConfigLen } {
       set tmpConfig [lindex $itk_option(-wdgConfig) $itk_option(-classSelect)] 
   } else {
       set tmpConfig [lindex $itk_option(-wdgConfig) end]
   }
       
   # Re-configure uifClass widget.
   if {$uifFlag && [winfo exists $myInstance] } {
       puts "eval $myInstance configure $tmpConfig"
       eval $myInstance configure $tmpConfig
   }
   
   $this configure -show 1
   $this show
   
}

#---------------------------------------------------------------
# METHOD : show
#---------------------------------------------------------------

body classContainer::show {} {

   if { [checkSelection] < 0 } { return }
   set myClass [lindex $itk_option(-wdgClassList) $itk_option(-classSelect)]
   set uifFlag [string match "*_uifClass" $myClass]

   wdgClassDisplay $myClass $uifFlag
   
   if {[uplevel #0 set $wthis.ckShow]} {
      $this display
      if {$isMapped && $uifFlag } { 
         $myInstance refreshInput
      }
   } else {
      $this hide
   }
}

#--------------------------------------------------------------------
#  OPTION : -showCommand 
#--------------------------------------------------------------------

configbody classContainer::showCommand {
   set tmp "$wthis setShow" 

   if {![cequal $itk_option(-showCommand) ""]} {
      append tmp "; $itk_option(-showCommand) $wthis" 
   } 
   $itk_component(ck) configure -command $tmp
}

#--------------------------------------------------------------------
#  OPTION : -titleList
#--------------------------------------------------------------------

configbody classContainer::titleList {

    # Destroy previus list of radioButtons ... if any

    foreach w [winfo children $itk_component(rbBox)] {
	destroy $w ;# Destroy previus list of radioButtons ... if any
    }

    if {[set lenTitle [llength $itk_option(-titleList)]] == 0} { return }

    if { $lenTitle == 1 } {
       # Configure as "singleContainer"
       $itk_component(title) configure -text [lindex $itk_option(-titleList) 0]
    } else {
       # Configure as "multiContainer"
       $itk_component(title) configure -text "Show -"

       for {set n 0} {$n < $lenTitle} {incr n} {
	   UifRadioButton  $itk_component(rbBox).rb${n} \
	                   -label [lindex $itk_option(-titleList) $n] \
	                   -onVal $n  -anchor nw \
                           -variable $wthis.rbChoice \
                           -command "$this configure -classSelect $n"

            pack $itk_component(rbBox).rb${n} -side left -padx 2
       }
    }
    if {$initialized} { setFont }
}

#___oOo___
