#*************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: classFrame.tcl,v 3.49 2005/09/07 11:45:58 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# santin    08/01/96  created
#                     basic code from uvesSingleContainer class
# mcomin    12/01/96  Review of code and man page
# mcomin    15/01/96  Add check if class is uifClass or not
#                     wdgLibrary supports list of libs.
# mcomin    22/01/96  Fixed bug in wdgLibrary pub. var.
# mcomin    09/02/96  Improved version after testing with CCD
# mcomin    02/04/96  Ported to Tk4.0 and itcl 2.0
#           08/05/96  Add missing options : relief, borderwidth
#           13/05/96  Update man page
#
#************************************************************************
#   NAME
#     classFrame - Frame for classe
#   SYNOPSIS
#     clssFrame <widget-name>
#   DESCRIPTION
#     The classFrame is a widget that can be configured to display a class 
#     out of a set of classes.
#
#     It does not support any interactive operation and the selection of the
#     class to be displayed is done programmatically.
#
#     Length and height are used when no class is visible in order to keep
#     its dimensions , otherwise the container assumes the size of the 
#     included class.
#   COMPONENTS
#
#   METHODS
#     The class inherits the methods of classCommon.
#   PUBLIC VARIABLES
#
#   WIDGET OPTIONS
#     The class inherits the widget options of classCommon and supports
#     the followings :
#
#     variable  <val>  Name of the variable used to set the class to
#                      be displayed.
#   CAUTIONS
#     Public variable wdgConfig applies to all instances of the classes,
#     therefore the same configuration string is executed whenever a class 
#     is created.
#   EXAMPLES
#
#     Create an instance
#
#     classFrame .w -background red -wdgLibrary libtestClass \
#                   -wdgClassList {xx_uifClass yy_uifClass zz_uifClass}\
#                   -wdgConfig    { {-cwp a:b} {-cwp a:c} {-cwp a:d} } \
#                   -variable gvar(selectFrame)
#      
#     Re-configure the widget
#        .w configure -classSelect 2      Display class #2
#        .w configure -show 0             Set visibility OFF
#        .w configure -background blue    Change color
#
#     Other operations
#        .w getClassPath                   Get full class path name
#        set list [.w cget -wdgClassList]  Get a widget's resource
#   SEE ALSO
#     classCommon  classContainer
#   BUGS
#
#------------------------------------------------------------------------
#

itk::usual classFrame { }

class classFrame {
   inherit classCommon
   
   constructor {args} {}
   destructor  {}

   itk_option define -variable variable Variable ""

   public  method show {}
   private method varTraceOff {}
   public  method varTrace    {name elem op}

   private variable oldVar  ""
}; # end of class definition


#------------------------------------------------------
# Constructor
#------------------------------------------------------

body classFrame::constructor {args} {

   itk_component add frame {
      frame $itk_interior.frame 
   } {     
      rename -background -classBackground classBackground ClassBackground
      keep -height -width -relief -borderwidth
   }

   uplevel #0 set $wthis.ckShow 0
   set frameName $itk_component(frame)

   pack $itk_component(frame) -pady 3 -side top -fill both

   # This option must be processed first to know specific class configurations

   if {[set idx [lsearch $args "-wdgConfig"]] != -1} {
       configure -wdgConfig [lindex $args [expr {$idx +1}]]
       set args [lreplace $args $idx [expr {$idx +1}]]
   }

   eval itk_initialize -height 20 -width 50 -relief groove -borderwidth 2 $args
   set initialized 1
}

#------------------------------------------------------
# Destructor
#------------------------------------------------------

body classFrame::destructor {} {
   if {![cequal $oldVar ""]} { varTraceOff }
   update
}

#---------------------------------------------------------------
# METHOD : varTraceOff - Disable callback
#---------------------------------------------------------------

body classFrame::varTraceOff { } {
#   puts "DBG > varTraceOff $this .. trace off !!"
   uplevel #0 trace vdelete $oldVar w  \"$wthis varTrace\"
}

#---------------------------------------------------------------
# METHOD : varTrace
#---------------------------------------------------------------

body classFrame::varTrace { name elem  op } {
   if {[cequal $elem ""]} {
      $this configure -classSelect [uplevel #0 set ${name}]
   } else {
      $this configure -classSelect [uplevel #0 set ${name}($elem)]
   }
#   puts "DBG > varTrace $this name = $name  elem = $elem  val = $refValue"
}

#---------------------------------------------------------------
# OPTION : -variable
#---------------------------------------------------------------

configbody  classFrame::variable {
   if {![cequal $itk_option(-variable) ""]} {
      if {! [uplevel #0 info exists $itk_option(-variable)]} {
         uplevel #0 set $itk_option(-variable) 0
      }

      $this configure -classSelect [uplevel #0 set $itk_option(-variable)]
      uplevel #0 trace variable $itk_option(-variable) w \"$wthis varTrace\"
   }

   if {![cequal $oldVar ""] && ![cequal $itk_option(-variable) $oldVar]} { 
      varTraceOff 
   }
   set oldVar $itk_option(-variable)
}


#---------------------------------------------------------------
# METHOD : show
#---------------------------------------------------------------

body classFrame::show {} { 

   if { [checkSelection] < 0 } { return }

   set myClass [lindex $itk_option(-wdgClassList) $itk_option(-classSelect)]
   set uifFlag [string match "*_uifClass" $myClass]

   wdgClassDisplay $myClass $uifFlag

   if {[uplevel #0 set $wthis.ckShow]} {
      $this display
      if { $isMapped && $uifFlag} { 
            $myInstance refreshInput
      }
   } else {
         $this hide
   }
}

#___oOo___

  
