#***********************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: wpStoreAccount.tcl,v 1.23 1998/03/09 08:55:29 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# mcomin    12/04/95  created
# gfilippi  31/05/95  set as a separate file from wpAccountLib.tcl
# gfilippi  01/06/95  SAVE does not ask for confirmation
# gfilippi  01/06/95  SAVE checks week and year
# gfilippi  08/06/95  do not save if activities are not in the archive
# gfilippi  09/01/96  correct file names for week 1-9  (01-09)
# mcomin    24/06/06  Modifiy call to short help area
#
#************************************************************************
#   NAME
#
#     wpStoreAccount - handle save into files SAVE & SUBMIT
#
#   DESCRIPTION
#
#     Internal procedure for 'Activity Account' panel (wpActivity.pan)
#
#------------------------------------------------------------------------

#------------------------------------------------------------------------
# PROCEDURE : wpStoreAccount  - Store current account to file
#------------------------------------------------------------------------
#   "Submit"
#       save the current total into a new time-sheet in the archive. 
#       save the current detailed data into local file.
#       The files are respectively named as
#
#              $wpAccountDir/<username>_<YY><WW>
#              $wpLocalAccDir/WP<username>_<YY><WW>
#
#       Once submitted, a time sheet cannot be modified.
#   
#   "Save"
#       save the current data into the user time-sheet:
#
#              $wpLocalFile
#       
#       (if not existing, $wpLocalAccDir is created)
#
#
# Common behaviour:
#
#    - if some of the activities are not present in the archive,
#      the user is prompted and no store is done.
#      This is to prevent that data contained into an old data sheet,
#      that has been loaded, can mess up the current databese. 
#     
#

proc wpStoreAccount { {what {}} {confirm {yes}} } {

   global cv wpUser wpAccountDir wpActivityDir wpLocalAccDir wpListWidget wpActRecord wpLocalFile
   global wp_name shArea gvar

   wpTotal
   
   #
   # check local account directory, if does not exist then create it
   #
   if {[file exists ${wpLocalAccDir}] == 0} {
       exec mkdir ${wpLocalAccDir}
       }

   #
   # check year and week
   #
   set week [$cv.week get]
   set year [$cv.year get]

   if { $week == "" || $week == "0" || $week > 53 } {
      set tmp "Wrong Week Number \n( valid range 1 - 53 ) "
      tk_dialog .dialog "Dialog Box" $tmp error 0 OK
      return
   }
      
   if { $year == "" || $year < 1994 || $year > 2010 } {
      set tmp "Wrong Year Number \n( valid range 1995 - 2010) "
      tk_dialog .dialog "Dialog Box" $tmp error 0 OK
      return
   }

   #
   # check that all the activities in the list are also in the database.
   #
   #
   set notInTheArchive ""
   for {set i 0} {$i < [$wpListWidget getSize]} {incr i} {
       set activity [lindex $wpActRecord($i) 0]
       if { [string length  $activity] == 0 } { continue }
       if { [file exists ${wpActivityDir}/$activity ] == 0} {
           lappend notInTheArchive $activity
       }
   }
   if { $notInTheArchive != ""} {
      set tmp "remove\n\n$notInTheArchive \n\nfrom the list because not in the archive \n(${wpActivityDir}) \n\nand $what again"
      tk_dialog .dialog "Dialog Box" $tmp error 0 OK
      return
   }

   #
   # format the week number into a 2digit string (1 --> 01)
   set WW [ format "%2.2d" $week ] 
   
   #
   # prepare the filename
   if { $what == "SUBMIT" } {
      set mode 0400   
      set filePath        "${wpLocalAccDir}/WP${wpUser}_${year}${WW}"
      set filePathArchive "${wpAccountDir}/${wpUser}_${year}${WW}"
      if {[file exists $filePathArchive] == 1} {
         set tmp "week ( ${WW} )\nALREADY ARCHIVED !!"
         tk_dialog .dialog "Dialog Box" $tmp info 0 OK
         return
      }
   } else {
      set mode 0600
      set filePath $wpLocalFile
   }

   #
   # open the file
   if { [catch { open $filePath "w+" $mode } fid] } {
      set tmp "Cannot open file : $filePath"
      $shArea displayText $tmp
      return
   }

   #
   # now that all the checks are done, ask user's confirmation
   if { ($what == "SUBMIT") } {
      set tmp "Do you want to SUBMIT \ncurrent Table ?"
      if {[tk_dialog .dialog "Dialog Box" $tmp question 1 Continue Cancel]} { 
         return 
      }
   }
   #
   # just to make the report nicer, evaluate the longest activity name
   #
   set max    [$wpListWidget getSize]
   set maxLen 0

   foreach member [array names wpActRecord] {
      set len [string length [getRecField $member $wp_name]]
      if { $len > $maxLen } { set maxLen $len } 
   }

   set myFormat "%-${maxLen}s   %4d %4d %4d %4d %4d %4d %4d   %4d"

   #
   # write header
   #
   puts $fid "WEEKLY TIME SHEET           User: ${wpUser}         Week: ${year}/${week}"
   if { $what == "SUBMIT" } {
      puts $fid ""
   } else {
      puts $fid "              --- TEMPORARY DATA - NOT YET SUBMITTED ---"
   }
   puts $fid [format "%-${maxLen}s     Mon  Tue  Wed  Thu  Fri  Sat  Sun   Total" "----Activities" ]
   puts $fid ""


   #
   # write table content 
   # (also the number of activities with time !=0 is counted)
   set myFormat "%-${maxLen}s   %4d %4d %4d %4d %4d %4d %4d   %4d"

   set doneActivity 0
   for {set i 0} {$i < $max} {incr i} {
       set tmp $wpActRecord($i)
       if { [lindex $tmp 8] == 0 && $what == "SUBMIT" } { continue }

       set line [format $myFormat [lindex $tmp 0] \
                                  [lindex $tmp 1] [lindex $tmp 2] \
                                  [lindex $tmp 3] [lindex $tmp 4] \
                                  [lindex $tmp 5] [lindex $tmp 6] \
                                  [lindex $tmp 7] [lindex $tmp 8] ]
       incr doneActivity 1
       puts $fid $line
   }

   # Write totals

   puts $fid "\n"
   set line [format $myFormat "Total" \
                        $gvar(col_1) $gvar(col_2) $gvar(col_3) \
                        $gvar(col_4) $gvar(col_5) $gvar(col_6) \
                        $gvar(col_7) $gvar(totWeek) ]
   puts $fid $line

   close $fid

   if {$what == "SUBMIT" } {

       # 
       # Save the summary value into the archive:
       # a special program is called. Such a program
       # has the capability to change UID and therefore
       # to become authorized to write into the protected archive
       
       set command "wpSendToArchive ${filePathArchive} $doneActivity "
      
       for {set i 0} {$i < $max} {incr i} {
           set tmp $wpActRecord($i)
           if { [lindex $tmp 8] != 0 } { 
               append  command  " "
               append  command  [lindex $tmp 0] 
               append  command  " "
               append  command  [lindex $tmp 8]
           }
       }
       
       eval exec $command

       # 
       # Save current value as the template for the coming week 
       incr week
       if { $week > 52 } { set week 1 ; incr year }
       wpClear
       set gvar(week) $week
       set gvar(year) $year
       
       wpStoreAccount SAVE no  
   }      

   $shArea displayText "Table SAVED into file : [file tail $filePath]"
}

#
# ___oOo___
